/**
 * @file
 * VuoPnpId implementation.
 *
 * @copyright Copyright © 2012–2018 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "module.h"
#include "VuoPnpId.h"

#include <string.h>

#ifdef VUO_COMPILER
VuoModuleMetadata({
					  "title" : "Plug and Play IDs",
					  "dependencies" : [
						  "VuoText"
					  ]
				 });
#endif

/**
 * Returns a string providing the full manufacturer name for the specified abbreviated 3-letter name.
 *
 * The PNP ID should be provided as a 15-bit value (3 x 5-bit values, where 1 = 'A').
 *
 * The caller is responsible for freeing the returned string.
 */
char *VuoPnpId_getString(uint16_t manufacturerId)
{
	char c0 = 'A' - 1 + ((manufacturerId >> 10) & 0x1f);
	char c1 = 'A' - 1 + ((((manufacturerId >> 8) & 0x03) << 3) | ((manufacturerId & 0xe0) >> 5));
	char c2 = 'A' - 1 + (manufacturerId & 0x1f);
	uint32_t fourCC = (' ' << 24) + (c0 << 16) + (c1 << 8) + c2;

	switch (fourCC)
	{
		// ========================================================
		// Exported http://www.uefi.org/uefi-pnp-export as TSV;
		// cat pnp.tsv | awk '{ print toupper(substr($0,1,3)) substr($0,4) }' | iconv -f MAC -t UTF8 | perl -pe 's/^([A-Z@]{3}).*\t"?([^"\n]+)"?$/case '"'"' $1'"'"': return strdup("$2");/' | sort
		// Why does "Vision Quest" have an "@" in its ID?
		case ' AAA': return strdup("Avolites Ltd");
		case ' AAE': return strdup("Anatek Electronics Inc.");
		case ' AAM': return strdup("Aava Mobile Oy");
		case ' AAN': return strdup("AAEON Technology Inc.");
		case ' AAT': return strdup("Ann Arbor Technologies");
		case ' ABA': return strdup("ABBAHOME INC.");
		case ' ABC': return strdup("AboCom System Inc.");
		case ' ABD': return strdup("Allen Bradley Company");
		case ' ABE': return strdup("Alcatel Bell");
		case ' ABO': return strdup("D-Link Systems Inc");
		case ' ABS': return strdup("Abaco Systems, Inc.");
		case ' ABT': return strdup("Anchor Bay Technologies, Inc.");
		case ' ABV': return strdup("Advanced Research Technology");
		case ' ACA': return strdup("Ariel Corporation");
		case ' ACB': return strdup("Aculab Ltd");
		case ' ACC': return strdup("Accton Technology Corporation");
		case ' ACD': return strdup("AWETA BV");
		case ' ACE': return strdup("Actek Engineering Pty Ltd");
		case ' ACG': return strdup("A&R Cambridge Ltd.");
		case ' ACH': return strdup("Archtek Telecom Corporation");
		case ' ACI': return strdup("Ancor Communications Inc");
		case ' ACK': return strdup("Acksys");
		case ' ACL': return strdup("Apricot Computers");
		case ' ACM': return strdup("Acroloop Motion Control Systems Inc");
		case ' ACO': return strdup("Allion Computer Inc.");
		case ' ACP': return strdup("Aspen Tech Inc");
		case ' ACR': return strdup("Acer Technologies");
		case ' ACS': return strdup("Altos Computer Systems");
		case ' ACT': return strdup("Applied Creative Technology");
		case ' ACU': return strdup("Acculogic");
		case ' ACV': return strdup("ActivCard S.A");
		case ' ADA': return strdup("Addi-Data GmbH");
		case ' ADB': return strdup("Aldebbaron");
		case ' ADC': return strdup("Acnhor Datacomm");
		case ' ADD': return strdup("Advanced Peripheral Devices Inc");
		case ' ADE': return strdup("Arithmos, Inc.");
		case ' ADH': return strdup("Aerodata Holdings Ltd");
		case ' ADI': return strdup("ADI Systems Inc");
		case ' ADK': return strdup("Adtek System Science Company Ltd");
		case ' ADL': return strdup("ASTRA Security Products Ltd");
		case ' ADM': return strdup("Ad Lib MultiMedia Inc");
		case ' ADN': return strdup("Analog & Digital Devices Tel. Inc");
		case ' ADP': return strdup("Adaptec Inc");
		case ' ADR': return strdup("Nasa Ames Research Center");
		case ' ADS': return strdup("Analog Devices Inc");
		case ' ADT': return strdup("Adtek");
		case ' ADV': return strdup("Advanced Micro Devices Inc");
		case ' ADX': return strdup("Adax Inc");
		case ' ADZ': return strdup("ADDER TECHNOLOGY LTD");
		case ' AEC': return strdup("Antex Electronics Corporation");
		case ' AED': return strdup("Advanced Electronic Designs, Inc.");
		case ' AEI': return strdup("Actiontec Electric Inc");
		case ' AEJ': return strdup("Alpha Electronics Company");
		case ' AEM': return strdup("ASEM S.p.A.");
		case ' AEN': return strdup("Avencall");
		case ' AEP': return strdup("Aetas Peripheral International");
		case ' AET': return strdup("Aethra Telecomunicazioni S.r.l.");
		case ' AFA': return strdup("Alfa Inc");
		case ' AGC': return strdup("Beijing Aerospace Golden Card Electronic Engineering Co.,Ltd.");
		case ' AGI': return strdup("Artish Graphics Inc");
		case ' AGL': return strdup("Argolis");
		case ' AGM': return strdup("Advan Int'l Corporation");
		case ' AGO': return strdup("AlgolTek, Inc.");
		case ' AGT': return strdup("Agilent Technologies");
		case ' AHC': return strdup("Advantech Co., Ltd.");
		case ' AHS': return strdup("Beijing AnHeng SecoTech Information Technology Co., Ltd.");
		case ' AIC': return strdup("Arnos Insturments & Computer Systems");
		case ' AIE': return strdup("Altmann Industrieelektronik");
		case ' AII': return strdup("Amptron International Inc.");
		case ' AIK': return strdup("Dongguan Alllike Electronics Co., Ltd.");
		case ' AIL': return strdup("Altos India Ltd");
		case ' AIM': return strdup("AIMS Lab Inc");
		case ' AIR': return strdup("Advanced Integ. Research Inc");
		case ' AIS': return strdup("Alien Internet Services");
		case ' AIW': return strdup("Aiwa Company Ltd");
		case ' AIX': return strdup("ALTINEX, INC.");
		case ' AJA': return strdup("AJA Video Systems, Inc.");
		case ' AKB': return strdup("Akebia Ltd");
		case ' AKE': return strdup("AKAMI Electric Co.,Ltd");
		case ' AKI': return strdup("AKIA Corporation");
		case ' AKL': return strdup("AMiT Ltd");
		case ' AKM': return strdup("Asahi Kasei Microsystems Company Ltd");
		case ' AKP': return strdup("Atom Komplex Prylad");
		case ' AKY': return strdup("Askey Computer Corporation");
		case ' ALA': return strdup("Alacron Inc");
		case ' ALC': return strdup("Altec Corporation");
		case ' ALD': return strdup("In4S Inc");
		case ' ALE': return strdup("Alenco BV");
		case ' ALG': return strdup("Realtek Semiconductor Corp.");
		case ' ALH': return strdup("AL Systems");
		case ' ALI': return strdup("Acer Labs");
		case ' ALJ': return strdup("Altec Lansing");
		case ' ALK': return strdup("Acrolink Inc");
		case ' ALL': return strdup("Alliance Semiconductor Corporation");
		case ' ALM': return strdup("Acutec Ltd.");
		case ' ALN': return strdup("Alana Technologies");
		case ' ALO': return strdup("Algolith Inc.");
		case ' ALP': return strdup("Alps Electric Company Ltd");
		case ' ALR': return strdup("Advanced Logic");
		case ' ALS': return strdup("Avance Logic Inc");
		case ' ALT': return strdup("Altra");
		case ' ALV': return strdup("AlphaView LCD");
		case ' ALX': return strdup("ALEXON Co.,Ltd.");
		case ' AMA': return strdup("Asia Microelectronic Development Inc");
		case ' AMB': return strdup("Ambient Technologies, Inc.");
		case ' AMC': return strdup("Attachmate Corporation");
		case ' AMD': return strdup("Amdek Corporation");
		case ' AMI': return strdup("American Megatrends Inc");
		case ' AML': return strdup("Anderson Multimedia Communications (HK) Limited");
		case ' AMN': return strdup("Amimon LTD.");
		case ' AMO': return strdup("Amino Technologies PLC and Amino Communications Limited");
		case ' AMP': return strdup("AMP Inc");
		case ' AMR': return strdup("AmTRAN Technology Co., Ltd.");
		case ' AMS': return strdup("ARMSTEL, Inc.");
		case ' AMT': return strdup("AMT International Industry");
		case ' AMX': return strdup("AMX LLC");
		case ' ANA': return strdup("Anakron");
		case ' ANC': return strdup("Ancot");
		case ' AND': return strdup("Adtran Inc");
		case ' ANI': return strdup("Anigma Inc");
		case ' ANK': return strdup("Anko Electronic Company Ltd");
		case ' ANL': return strdup("Analogix Semiconductor, Inc");
		case ' ANO': return strdup("Anorad Corporation");
		case ' ANP': return strdup("Andrew Network Production");
		case ' ANR': return strdup("ANR Ltd");
		case ' ANS': return strdup("Ansel Communication Company");
		case ' ANT': return strdup("Ace CAD Enterprise Company Ltd");
		case ' ANV': return strdup("Beijing ANTVR Technology Co., Ltd.");
		case ' ANW': return strdup("Analog Way SAS");
		case ' ANX': return strdup("Acer Netxus Inc");
		case ' AOA': return strdup("AOpen Inc.");
		case ' AOE': return strdup("Advanced Optics Electronics, Inc.");
		case ' AOL': return strdup("America OnLine");
		case ' AOT': return strdup("Alcatel");
		case ' APC': return strdup("American Power Conversion");
		case ' APD': return strdup("AppliAdata");
		case ' APE': return strdup("Alpine Electronics, Inc.");
		case ' APG': return strdup("Horner Electric Inc");
		case ' API': return strdup("A Plus Info Corporation");
		case ' APL': return strdup("Aplicom Oy");
		case ' APM': return strdup("Applied Memory Tech");
		case ' APN': return strdup("Appian Tech Inc");
		case ' APP': return strdup("Apple Computer Inc");
		case ' APR': return strdup("Aprilia s.p.a.");
		case ' APS': return strdup("Autologic Inc");
		case ' APT': return strdup("Audio Processing Technology Ltd");
		case ' APV': return strdup("A+V Link");
		case ' APX': return strdup("AP Designs Ltd");
		case ' ARC': return strdup("Alta Research Corporation");
		case ' ARD': return strdup("AREC Inc.");
		case ' ARE': return strdup("ICET S.p.A.");
		case ' ARG': return strdup("Argus Electronics Co., LTD");
		case ' ARI': return strdup("Argosy Research Inc");
		case ' ARK': return strdup("Ark Logic Inc");
		case ' ARL': return strdup("Arlotto Comnet Inc");
		case ' ARM': return strdup("Arima");
		case ' ARO': return strdup("Poso International B.V.");
		case ' ARR': return strdup("ARRIS Group, Inc.");
		case ' ARS': return strdup("Arescom Inc");
		case ' ART': return strdup("Corion Industrial Corporation");
		case ' ASC': return strdup("Ascom Strategic Technology Unit");
		case ' ASD': return strdup("USC Information Sciences Institute");
		case ' ASE': return strdup("AseV Display Labs");
		case ' ASH': return strdup("Ashton Bentley Concepts");
		case ' ASI': return strdup("Ahead Systems");
		case ' ASK': return strdup("Ask A/S");
		case ' ASL': return strdup("AccuScene Corporation Ltd");
		case ' ASM': return strdup("ASEM S.p.A.");
		case ' ASN': return strdup("Asante Tech Inc");
		case ' ASP': return strdup("ASP Microelectronics Ltd");
		case ' AST': return strdup("AST Research Inc");
		case ' ASU': return strdup("Asuscom Network Inc");
		case ' ASX': return strdup("AudioScience");
		case ' ASY': return strdup("Rockwell Collins / Airshow Systems");
		case ' ATA': return strdup("Allied Telesyn International (Asia) Pte Ltd");
		case ' ATC': return strdup("Ably-Tech Corporation");
		case ' ATD': return strdup("Alpha Telecom Inc");
		case ' ATE': return strdup("Innovate Ltd");
		case ' ATH': return strdup("Athena Informatica S.R.L.");
		case ' ATI': return strdup("Allied Telesis KK");
		case ' ATJ': return strdup("ArchiTek Corporation");
		case ' ATK': return strdup("Allied Telesyn Int'l");
		case ' ATL': return strdup("Arcus Technology Ltd");
		case ' ATM': return strdup("ATM Ltd");
		case ' ATN': return strdup("Athena Smartcard Solutions Ltd.");
		case ' ATO': return strdup("ASTRO DESIGN, INC.");
		case ' ATP': return strdup("Alpha-Top Corporation");
		case ' ATT': return strdup("AT&T");
		case ' ATV': return strdup("Office Depot, Inc.");
		case ' ATX': return strdup("Athenix Corporation");
		case ' AUG': return strdup("August Home, Inc.");
		case ' AUI': return strdup("Alps Electric Inc");
		case ' AUO': return strdup("DO NOT USE - AUO");
		case ' AUR': return strdup("Aureal Semiconductor");
		case ' AUS': return strdup("ASUSTek COMPUTER INC");
		case ' AUT': return strdup("Autotime Corporation");
		case ' AUV': return strdup("Auvidea GmbH");
		case ' AVA': return strdup("Avaya Communication");
		case ' AVC': return strdup("Auravision Corporation");
		case ' AVD': return strdup("Avid Electronics Corporation");
		case ' AVE': return strdup("Add Value Enterpises (Asia) Pte Ltd");
		case ' AVG': return strdup("Avegant Corporation");
		case ' AVI': return strdup("Nippon Avionics Co.,Ltd");
		case ' AVJ': return strdup("Atelier Vision Corporation");
		case ' AVL': return strdup("Avalue Technology Inc.");
		case ' AVM': return strdup("AVM GmbH");
		case ' AVN': return strdup("Advance Computer Corporation");
		case ' AVO': return strdup("Avocent Corporation");
		case ' AVR': return strdup("AVer Information Inc.");
		case ' AVT': return strdup("Avtek (Electronics) Pty Ltd");
		case ' AVV': return strdup("SBS Technologies (Canada), Inc. (was Avvida Systems, Inc.)");
		case ' AVX': return strdup("A/Vaux Electronics");
		case ' AWC': return strdup("Access Works Comm Inc");
		case ' AWL': return strdup("Aironet Wireless Communications, Inc");
		case ' AWS': return strdup("Wave Systems");
		case ' AXB': return strdup("Adrienne Electronics Corporation");
		case ' AXC': return strdup("AXIOMTEK CO., LTD.");
		case ' AXE': return strdup("Axell Corporation");
		case ' AXI': return strdup("American Magnetics");
		case ' AXL': return strdup("Axel");
		case ' AXO': return strdup("Axonic Labs LLC");
		case ' AXP': return strdup("American Express");
		case ' AXT': return strdup("Axtend Technologies Inc");
		case ' AXX': return strdup("Axxon Computer Corporation");
		case ' AXY': return strdup("AXYZ Automation Services, Inc");
		case ' AYD': return strdup("Aydin Displays");
		case ' AYR': return strdup("Airlib, Inc");
		case ' AZH': return strdup("Shenzhen three Connaught Information Technology Co., Ltd. (3nod Group)");
		case ' AZM': return strdup("AZ Middelheim - Radiotherapy");
		case ' AZT': return strdup("Aztech Systems Ltd");
		case ' BAC': return strdup("Biometric Access Corporation");
		case ' BAN': return strdup("Banyan");
		case ' BBB': return strdup("an-najah university");
		case ' BBH': return strdup("B&Bh");
		case ' BBL': return strdup("Brain Boxes Limited");
		case ' BCC': return strdup("Beaver Computer Corporaton");
		case ' BCD': return strdup("Barco GmbH");
		case ' BCI': return strdup("Broadata Communications Inc.");
		case ' BCM': return strdup("Broadcom");
		case ' BCQ': return strdup("Deutsche Telekom Berkom GmbH");
		case ' BCS': return strdup("Booria CAD/CAM systems");
		case ' BDO': return strdup("Brahler ICS");
		case ' BDR': return strdup("Blonder Tongue Labs, Inc.");
		case ' BDS': return strdup("Barco Display Systems");
		case ' BEC': return strdup("Beckhoff Automation");
		case ' BEI': return strdup("Beckworth Enterprises Inc");
		case ' BEK': return strdup("Beko Elektronik A.S.");
		case ' BEL': return strdup("Beltronic Industrieelektronik GmbH");
		case ' BEO': return strdup("Baug & Olufsen");
		case ' BFE': return strdup("B.F. Engineering Corporation");
		case ' BGB': return strdup("Barco Graphics N.V");
		case ' BGT': return strdup("Budzetron Inc");
		case ' BHZ': return strdup("BitHeadz, Inc.");
		case ' BIA': return strdup("Biamp Systems Corporation");
		case ' BIC': return strdup("Big Island Communications");
		case ' BII': return strdup("Boeckeler Instruments Inc");
		case ' BIL': return strdup("Billion Electric Company Ltd");
		case ' BIO': return strdup("BioLink Technologies International, Inc.");
		case ' BIT': return strdup("Bit 3 Computer");
		case ' BLI': return strdup("Busicom");
		case ' BLN': return strdup("BioLink Technologies");
		case ' BLP': return strdup("Bloomberg L.P.");
		case ' BMD': return strdup("Blackmagic Design");
		case ' BMI': return strdup("Benson Medical Instruments Company");
		case ' BML': return strdup("BIOMED Lab");
		case ' BMS': return strdup("BIOMEDISYS");
		case ' BNE': return strdup("Bull AB");
		case ' BNK': return strdup("Banksia Tech Pty Ltd");
		case ' BNO': return strdup("Bang & Olufsen");
		case ' BNS': return strdup("Boulder Nonlinear Systems");
		case ' BOB': return strdup("Rainy Orchard");
		case ' BOE': return strdup("BOE");
		case ' BOI': return strdup("NINGBO BOIGLE DIGITAL TECHNOLOGY CO.,LTD");
		case ' BOS': return strdup("BOS");
		case ' BPD': return strdup("Micro Solutions, Inc.");
		case ' BPS': return strdup("Barco, N.V.");
		case ' BPU': return strdup("Best Power");
		case ' BRA': return strdup("Braemac Pty Ltd");
		case ' BRC': return strdup("BARC");
		case ' BRG': return strdup("Bridge Information Co., Ltd");
		case ' BRI': return strdup("Boca Research Inc");
		case ' BRM': return strdup("Braemar Inc");
		case ' BRO': return strdup("BROTHER INDUSTRIES,LTD.");
		case ' BSE': return strdup("Bose Corporation");
		case ' BSG': return strdup("Robert Bosch GmbH");
		case ' BSL': return strdup("Biomedical Systems Laboratory");
		case ' BSN': return strdup("BRIGHTSIGN, LLC");
		case ' BST': return strdup("BodySound Technologies, Inc.");
		case ' BTC': return strdup("Bit 3 Computer");
		case ' BTE': return strdup("Brilliant Technology");
		case ' BTF': return strdup("Bitfield Oy");
		case ' BTI': return strdup("BusTech Inc");
		case ' BTO': return strdup("BioTao Ltd");
		case ' BUF': return strdup("Yasuhiko Shirai Melco Inc");
		case ' BUG': return strdup("B.U.G., Inc.");
		case ' BUJ': return strdup("ATI Tech Inc");
		case ' BUL': return strdup("Bull");
		case ' BUR': return strdup("Bernecker & Rainer Ind-Eletronik GmbH");
		case ' BUS': return strdup("BusTek");
		case ' BUT': return strdup("21ST CENTURY ENTERTAINMENT");
		case ' BWK': return strdup("Bitworks Inc.");
		case ' BXE': return strdup("Buxco Electronics");
		case ' BYD': return strdup("byd:sign corporation");
		case ' CAA': return strdup("Castles Automation Co., Ltd");
		case ' CAC': return strdup("CA & F Elettronica");
		case ' CAG': return strdup("CalComp");
		case ' CAI': return strdup("Canon Inc.");
		case ' CAL': return strdup("Acon");
		case ' CAM': return strdup("Cambridge Audio");
		case ' CAN': return strdup("Canopus Company Ltd");
		case ' CAR': return strdup("Cardinal Company Ltd");
		case ' CAS': return strdup("CASIO COMPUTER CO.,LTD");
		case ' CAT': return strdup("Consultancy in Advanced Technology");
		case ' CAV': return strdup("Cavium Networks, Inc");
		case ' CBI': return strdup("ComputerBoards Inc");
		case ' CBR': return strdup("Cebra Tech A/S");
		case ' CBT': return strdup("Cabletime Ltd");
		case ' CBX': return strdup("Cybex Computer Products Corporation");
		case ' CCC': return strdup("C-Cube Microsystems");
		case ' CCI': return strdup("Cache");
		case ' CCJ': return strdup("CONTEC CO.,LTD.");
		case ' CCL': return strdup("CCL/ITRI");
		case ' CCP': return strdup("Capetronic USA Inc");
		case ' CDC': return strdup("Core Dynamics Corporation");
		case ' CDD': return strdup("Convergent Data Devices");
		case ' CDE': return strdup("Colin.de");
		case ' CDG': return strdup("Christie Digital Systems Inc");
		case ' CDI': return strdup("Concept Development Inc");
		case ' CDK': return strdup("Cray Communications");
		case ' CDN': return strdup("Codenoll Technical Corporation");
		case ' CDP': return strdup("CalComp");
		case ' CDS': return strdup("Computer Diagnostic Systems");
		case ' CDT': return strdup("IBM Corporation");
		case ' CDV': return strdup("Convergent Design Inc.");
		case ' CEA': return strdup("Consumer Electronics Association");
		case ' CEC': return strdup("Chicony Electronics Company Ltd");
		case ' CED': return strdup("Cambridge Electronic Design Ltd");
		case ' CEF': return strdup("Cefar Digital Vision");
		case ' CEI': return strdup("Crestron Electronics, Inc.");
		case ' CEM': return strdup("MEC Electronics GmbH");
		case ' CEN': return strdup("Centurion Technologies P/L");
		case ' CEP': return strdup("C-DAC");
		case ' CER': return strdup("Ceronix");
		case ' CET': return strdup("TEC CORPORATION");
		case ' CFG': return strdup("Atlantis");
		case ' CGA': return strdup("Chunghwa Picture Tubes, LTD");
		case ' CGS': return strdup("Chyron Corp");
		case ' CGT': return strdup("congatec AG");
		case ' CHA': return strdup("Chase Research PLC");
		case ' CHD': return strdup("ChangHong Electric Co.,Ltd");
		case ' CHE': return strdup("Acer Inc");
		case ' CHG': return strdup("Sichuan Changhong Electric CO, LTD.");
		case ' CHI': return strdup("Chrontel Inc");
		case ' CHL': return strdup("Chloride-R&D");
		case ' CHM': return strdup("CHIC TECHNOLOGY CORP.");
		case ' CHO': return strdup("Sichuang Changhong Corporation");
		case ' CHP': return strdup("CH Products");
		case ' CHS': return strdup("Agentur Chairos");
		case ' CHT': return strdup("Chunghwa Picture Tubes,LTD.");
		case ' CHY': return strdup("Cherry GmbH");
		case ' CIC': return strdup("Comm. Intelligence Corporation");
		case ' CII': return strdup("Cromack Industries Inc");
		case ' CIL': return strdup("Citicom Infotech Private Limited");
		case ' CIN': return strdup("Citron GmbH");
		case ' CIP': return strdup("Ciprico Inc");
		case ' CIR': return strdup("Cirrus Logic Inc");
		case ' CIS': return strdup("Cisco Systems Inc");
		case ' CIT': return strdup("Citifax Limited");
		case ' CKC': return strdup("The Concept Keyboard Company Ltd");
		case ' CKJ': return strdup("Carina System Co., Ltd.");
		case ' CLA': return strdup("Clarion Company Ltd");
		case ' CLD': return strdup("COMMAT L.t.d.");
		case ' CLE': return strdup("Classe Audio");
		case ' CLG': return strdup("CoreLogic");
		case ' CLI': return strdup("Cirrus Logic Inc");
		case ' CLM': return strdup("CrystaLake Multimedia");
		case ' CLO': return strdup("Clone Computers");
		case ' CLT': return strdup("automated computer control systems");
		case ' CLV': return strdup("Clevo Company");
		case ' CLX': return strdup("CardLogix");
		case ' CMC': return strdup("CMC Ltd");
		case ' CMD': return strdup("Colorado MicroDisplay, Inc.");
		case ' CMG': return strdup("Chenming Mold Ind. Corp.");
		case ' CMI': return strdup("C-Media Electronics");
		case ' CMM': return strdup("Comtime GmbH");
		case ' CMN': return strdup("Chimei Innolux Corporation");
		case ' CMO': return strdup("Chi Mei Optoelectronics corp.");
		case ' CMR': return strdup("Cambridge Research Systems Ltd");
		case ' CMS': return strdup("CompuMaster Srl");
		case ' CMX': return strdup("Comex Electronics AB");
		case ' CNB': return strdup("American Power Conversion");
		case ' CNC': return strdup("Alvedon Computers Ltd");
		case ' CNE': return strdup("Cine-tal");
		case ' CNI': return strdup("Connect Int'l A/S");
		case ' CNN': return strdup("Canon Inc");
		case ' CNT': return strdup("COINT Multimedia Systems");
		case ' COB': return strdup("COBY Electronics Co., Ltd");
		case ' COD': return strdup("CODAN Pty. Ltd.");
		case ' COI': return strdup("Codec Inc.");
		case ' COL': return strdup("Rockwell Collins, Inc.");
		case ' COM': return strdup("Comtrol Corporation");
		case ' CON': return strdup("Contec Company Ltd");
		case ' COO': return strdup("coolux GmbH");
		case ' COR': return strdup("Corollary Inc");
		case ' COS': return strdup("CoStar Corporation");
		case ' COT': return strdup("Core Technology Inc");
		case ' COW': return strdup("Polycow Productions");
		case ' COX': return strdup("Comrex");
		case ' CPC': return strdup("Ciprico Inc");
		case ' CPD': return strdup("CompuAdd");
		case ' CPI': return strdup("Computer Peripherals Inc");
		case ' CPL': return strdup("Compal Electronics Inc");
		case ' CPM': return strdup("Capella Microsystems Inc.");
		case ' CPP': return strdup("Compound Photonics");
		case ' CPQ': return strdup("Compaq Computer Company");
		case ' CPT': return strdup("cPATH");
		case ' CPX': return strdup("Powermatic Data Systems");
		case ' CRA': return strdup("CRALTECH ELECTRONICA, S.L.");
		case ' CRC': return strdup("CONRAC GmbH");
		case ' CRD': return strdup("Cardinal Technical Inc");
		case ' CRE': return strdup("Creative Labs Inc");
		case ' CRH': return strdup("Contemporary Research Corp.");
		case ' CRI': return strdup("Crio Inc.");
		case ' CRL': return strdup("Creative Logic  ");
		case ' CRN': return strdup("Cornerstone Imaging");
		case ' CRO': return strdup("Extraordinary Technologies PTY Limited");
		case ' CRQ': return strdup("Cirque Corporation");
		case ' CRS': return strdup("Crescendo Communication Inc");
		case ' CRV': return strdup("Cerevo Inc.");
		case ' CRX': return strdup("Cyrix Corporation");
		case ' CSB': return strdup("Transtex SA");
		case ' CSC': return strdup("Crystal Semiconductor");
		case ' CSD': return strdup("Cresta Systems Inc");
		case ' CSE': return strdup("Concept Solutions & Engineering");
		case ' CSI': return strdup("Cabletron System Inc");
		case ' CSL': return strdup("Cloudium Systems Ltd.");
		case ' CSM': return strdup("Cosmic Engineering Inc.");
		case ' CSO': return strdup("California Institute of Technology");
		case ' CSS': return strdup("CSS Laboratories");
		case ' CST': return strdup("CSTI Inc");
		case ' CTA': return strdup("CoSystems Inc");
		case ' CTC': return strdup("CTC Communication Development Company Ltd");
		case ' CTE': return strdup("Chunghwa Telecom Co., Ltd.");
		case ' CTL': return strdup("Creative Technology Ltd");
		case ' CTM': return strdup("Computerm Corporation");
		case ' CTN': return strdup("Computone Products");
		case ' CTP': return strdup("Computer Technology Corporation");
		case ' CTR': return strdup("Control4 Corporation");
		case ' CTS': return strdup("Comtec Systems Co., Ltd.");
		case ' CTX': return strdup("Creatix Polymedia GmbH");
		case ' CUB': return strdup("Cubix Corporation");
		case ' CUK': return strdup("Calibre UK Ltd");
		case ' CVA': return strdup("Covia Inc.");
		case ' CVI': return strdup("Colorado Video, Inc.");
		case ' CVP': return strdup("Chromatec Video Products Ltd");
		case ' CVS': return strdup("Clarity Visual Systems");
		case ' CWC': return strdup("Curtiss-Wright Controls, Inc.");
		case ' CWR': return strdup("Connectware Inc");
		case ' CXT': return strdup("Conexant Systems");
		case ' CYB': return strdup("CyberVision");
		case ' CYC': return strdup("Cylink Corporation");
		case ' CYD': return strdup("Cyclades Corporation");
		case ' CYL': return strdup("Cyberlabs");
		case ' CYP': return strdup("CYPRESS SEMICONDUCTOR CORPORATION");
		case ' CYT': return strdup("Cytechinfo Inc");
		case ' CYV': return strdup("Cyviz AS");
		case ' CYW': return strdup("Cyberware");
		case ' CYX': return strdup("Cyrix Corporation");
		case ' CZC': return strdup("Shenzhen ChuangZhiCheng Technology Co., Ltd.");
		case ' CZE': return strdup("Carl Zeiss AG");
		case ' DAC': return strdup("Digital Acoustics Corporation");
		case ' DAE': return strdup("Digatron Industrie Elektronik GmbH");
		case ' DAI': return strdup("DAIS SET Ltd.");
		case ' DAK': return strdup("Daktronics");
		case ' DAL': return strdup("Digital Audio Labs Inc");
		case ' DAN': return strdup("Danelec Marine A/S");
		case ' DAS': return strdup("DAVIS AS");
		case ' DAT': return strdup("Datel Inc");
		case ' DAU': return strdup("Daou Tech Inc");
		case ' DAV': return strdup("Davicom Semiconductor Inc");
		case ' DAW': return strdup("DA2 Technologies Inc");
		case ' DAX': return strdup("Data Apex Ltd");
		case ' DBD': return strdup("Diebold Inc.");
		case ' DBI': return strdup("DigiBoard Inc");
		case ' DBK': return strdup("Databook Inc");
		case ' DBL': return strdup("Doble Engineering Company");
		case ' DBN': return strdup("DB Networks Inc");
		case ' DCA': return strdup("Digital Communications Association");
		case ' DCC': return strdup("Dale Computer Corporation");
		case ' DCD': return strdup("Datacast LLC");
		case ' DCE': return strdup("dSPACE GmbH");
		case ' DCI': return strdup("Concepts Inc");
		case ' DCL': return strdup("Dynamic Controls Ltd");
		case ' DCM': return strdup("DCM Data Products");
		case ' DCO': return strdup("Dialogue Technology Corporation");
		case ' DCR': return strdup("Decros Ltd");
		case ' DCS': return strdup("Diamond Computer Systems Inc");
		case ' DCT': return strdup("Dancall Telecom A/S");
		case ' DCV': return strdup("Datatronics Technology Inc");
		case ' DDA': return strdup("DA2 Technologies Corporation");
		case ' DDD': return strdup("Danka Data Devices");
		case ' DDE': return strdup("Datasat Digital Entertainment");
		case ' DDI': return strdup("Data Display AG");
		case ' DDS': return strdup("Barco, N.V.");
		case ' DDT': return strdup("Datadesk Technologies Inc");
		case ' DDV': return strdup("Delta Information Systems, Inc");
		case ' DEC': return strdup("Digital Equipment Corporation");
		case ' DEI': return strdup("Deico Electronics");
		case ' DEL': return strdup("Dell Inc.");
		case ' DEN': return strdup("Densitron Computers Ltd");
		case ' DEX': return strdup("idex displays");
		case ' DFI': return strdup("DFI");
		case ' DFK': return strdup("SharkTec A/S");
		case ' DFT': return strdup("DEI Holdings dba Definitive Technology");
		case ' DGA': return strdup("Digiital Arts Inc");
		case ' DGC': return strdup("Data General Corporation");
		case ' DGI': return strdup("DIGI International");
		case ' DGK': return strdup("DugoTech Co., LTD");
		case ' DGP': return strdup("Digicorp European sales S.A.");
		case ' DGS': return strdup("Diagsoft Inc");
		case ' DGT': return strdup("Dearborn Group Technology");
		case ' DHD': return strdup("Dension Audio Systems");
		case ' DHP': return strdup("DH Print");
		case ' DHQ': return strdup("Quadram");
		case ' DHT': return strdup("Projectavision Inc");
		case ' DIA': return strdup("Diadem");
		case ' DIG': return strdup("Digicom S.p.A.");
		case ' DII': return strdup("Dataq Instruments Inc");
		case ' DIM': return strdup("dPict Imaging, Inc.");
		case ' DIN': return strdup("Daintelecom Co., Ltd");
		case ' DIS': return strdup("Diseda S.A.");
		case ' DIT': return strdup("Dragon Information Technology");
		case ' DJE': return strdup("Capstone Visua lProduct Development");
		case ' DJP': return strdup("Maygay Machines, Ltd");
		case ' DKY': return strdup("Datakey Inc");
		case ' DLB': return strdup("Dolby Laboratories Inc.");
		case ' DLC': return strdup("Diamond Lane Comm. Corporation");
		case ' DLG': return strdup("Digital-Logic GmbH");
		case ' DLK': return strdup("D-Link Systems Inc");
		case ' DLL': return strdup("Dell Inc");
		case ' DLT': return strdup("Digitelec Informatique Park Cadera");
		case ' DMB': return strdup("Digicom Systems Inc");
		case ' DMC': return strdup("Dune Microsystems Corporation");
		case ' DMM': return strdup("Dimond Multimedia Systems Inc");
		case ' DMO': return strdup("Data Modul AG");
		case ' DMP': return strdup("D&M Holdings Inc, Professional Business Company");
		case ' DMS': return strdup("DOME imaging systems");
		case ' DMT': return strdup("Distributed Management Task Force, Inc. (DMTF)");
		case ' DMV': return strdup("NDS Ltd");
		case ' DNA': return strdup("DNA Enterprises, Inc.");
		case ' DNG': return strdup("Apache Micro Peripherals Inc");
		case ' DNI': return strdup("Deterministic Networks Inc.");
		case ' DNT': return strdup("Dr. Neuhous Telekommunikation GmbH");
		case ' DNV': return strdup("DiCon");
		case ' DOL': return strdup("Dolman Technologies Group Inc");
		case ' DOM': return strdup("Dome Imaging Systems");
		case ' DON': return strdup("DENON, Ltd.");
		case ' DOT': return strdup("Dotronic Mikroelektronik GmbH");
		case ' DPA': return strdup("DigiTalk Pro AV");
		case ' DPC': return strdup("Delta Electronics Inc");
		case ' DPH': return strdup("Delphi Automotive LLP");
		case ' DPI': return strdup("DocuPoint");
		case ' DPL': return strdup("Digital Projection Limited");
		case ' DPM': return strdup("ADPM Synthesis sas");
		case ' DPS': return strdup("Digital Processing Systems");
		case ' DPT': return strdup("DPT");
		case ' DPX': return strdup("DpiX, Inc.");
		case ' DQB': return strdup("Datacube Inc");
		case ' DRB': return strdup("Dr. Bott KG");
		case ' DRC': return strdup("Data Ray Corp.");
		case ' DRD': return strdup("DIGITAL REFLECTION INC.");
		case ' DRI': return strdup("Data Race Inc");
		case ' DRS': return strdup("DRS Defense Solutions, LLC");
		case ' DSA': return strdup("Display Solution AG");
		case ' DSD': return strdup("DS Multimedia Pte Ltd");
		case ' DSI': return strdup("Digitan Systems Inc");
		case ' DSM': return strdup("DSM Digital Services GmbH");
		case ' DSP': return strdup("Domain Technology Inc");
		case ' DTA': return strdup("DELTATEC");
		case ' DTC': return strdup("DTC Tech Corporation");
		case ' DTE': return strdup("Dimension Technologies, Inc.");
		case ' DTI': return strdup("Diversified Technology, Inc.");
		case ' DTK': return strdup("Dynax Electronics (HK) Ltd");
		case ' DTL': return strdup("e-Net Inc");
		case ' DTN': return strdup("Datang Telephone Co");
		case ' DTO': return strdup("Deutsche Thomson OHG");
		case ' DTT': return strdup("Design & Test Technology, Inc.");
		case ' DTX': return strdup("Data Translation");
		case ' DUA': return strdup("Dosch & Amand GmbH & Company KG");
		case ' DUN': return strdup("NCR Corporation");
		case ' DVD': return strdup("Dictaphone Corporation");
		case ' DVL': return strdup("Devolo AG");
		case ' DVS': return strdup("Digital Video System");
		case ' DVT': return strdup("Data Video");
		case ' DWE': return strdup("Daewoo Electronics Company Ltd");
		case ' DXC': return strdup("Digipronix Control Systems");
		case ' DXD': return strdup("DECIMATOR DESIGN PTY LTD");
		case ' DXL': return strdup("Dextera Labs Inc");
		case ' DXP': return strdup("Data Expert Corporation");
		case ' DXS': return strdup("Signet");
		case ' DYC': return strdup("Dycam Inc");
		case ' DYM': return strdup("Dymo-CoStar Corporation");
		case ' DYN': return strdup("Askey Computer Corporation");
		case ' DYX': return strdup("Dynax Electronics (HK) Ltd");
		case ' EAG': return strdup("ELTEC Elektronik AG");
		case ' EAS': return strdup("Evans and Sutherland Computer");
		case ' EBH': return strdup("Data Price Informatica");
		case ' EBS': return strdup("EBS Euchner Büro- und Schulsysteme GmbH");
		case ' EBT': return strdup("HUALONG TECHNOLOGY CO., LTD");
		case ' ECA': return strdup("Electro Cam Corp.");
		case ' ECC': return strdup("ESSential Comm. Corporation");
		case ' ECH': return strdup("EchoStar Corporation");
		case ' ECI': return strdup("Enciris Technologies");
		case ' ECK': return strdup("Eugene Chukhlomin Sole Proprietorship, d.b.a.");
		case ' ECL': return strdup("Excel Company Ltd");
		case ' ECM': return strdup("E-Cmos Tech Corporation");
		case ' ECO': return strdup("Echo Speech Corporation");
		case ' ECP': return strdup("Elecom Company Ltd");
		case ' ECS': return strdup("Elitegroup Computer Systems Company Ltd");
		case ' ECT': return strdup("Enciris Technologies");
		case ' EDC': return strdup("e.Digital Corporation");
		case ' EDG': return strdup("Electronic-Design GmbH");
		case ' EDI': return strdup("Edimax Tech. Company Ltd");
		case ' EDM': return strdup("EDMI");
		case ' EDT': return strdup("Emerging Display Technologies Corp");
		case ' EEE': return strdup("ET&T Technology Company Ltd");
		case ' EEH': return strdup("EEH Datalink GmbH");
		case ' EEP': return strdup("E.E.P.D. GmbH");
		case ' EES': return strdup("EE Solutions, Inc.");
		case ' EGA': return strdup("Elgato Systems LLC");
		case ' EGD': return strdup("EIZO GmbH Display Technologies");
		case ' EGL': return strdup("Eagle Technology");
		case ' EGN': return strdup("Egenera, Inc.");
		case ' EGO': return strdup("Ergo Electronics");
		case ' EHJ': return strdup("Epson Research");
		case ' EHN': return strdup("Enhansoft");
		case ' EIC': return strdup("Eicon Technology Corporation");
		case ' EKA': return strdup("MagTek Inc.");
		case ' EKC': return strdup("Eastman Kodak Company");
		case ' EKS': return strdup("EKSEN YAZILIM");
		case ' ELA': return strdup("ELAD srl");
		case ' ELC': return strdup("Electro Scientific Ind");
		case ' ELE': return strdup("Elecom Company Ltd");
		case ' ELG': return strdup("Elmeg GmbH Kommunikationstechnik");
		case ' ELI': return strdup("Edsun Laboratories");
		case ' ELL': return strdup("Electrosonic Ltd");
		case ' ELM': return strdup("Elmic Systems Inc");
		case ' ELO': return strdup("Elo TouchSystems Inc");
		case ' ELS': return strdup("ELSA GmbH");
		case ' ELT': return strdup("Element Labs, Inc.");
		case ' ELU': return strdup("Express Industrial, Ltd.");
		case ' ELX': return strdup("Elonex PLC");
		case ' EMB': return strdup("Embedded computing inc ltd");
		case ' EMC': return strdup("eMicro Corporation");
		case ' EMD': return strdup("Embrionix Design Inc.");
		case ' EME': return strdup("EMiNE TECHNOLOGY COMPANY, LTD.");
		case ' EMG': return strdup("EMG Consultants Inc");
		case ' EMI': return strdup("Ex Machina Inc");
		case ' EMK': return strdup("Emcore Corporation");
		case ' EMO': return strdup("ELMO COMPANY, LIMITED");
		case ' EMU': return strdup("Emulex Corporation");
		case ' ENC': return strdup("Eizo Nanao Corporation");
		case ' END': return strdup("ENIDAN Technologies Ltd");
		case ' ENE': return strdup("ENE Technology Inc.");
		case ' ENI': return strdup("Efficient Networks");
		case ' ENS': return strdup("Ensoniq Corporation");
		case ' ENT': return strdup("Enterprise Comm. & Computing Inc");
		case ' EON': return strdup("Eon Instrumentation, Inc.");
		case ' EPC': return strdup("Empac");
		case ' EPH': return strdup("Epiphan Systems Inc. ");
		case ' EPI': return strdup("Envision Peripherals, Inc");
		case ' EPN': return strdup("EPiCON Inc.");
		case ' EPS': return strdup("KEPS");
		case ' EQP': return strdup("Equipe Electronics Ltd.");
		case ' EQX': return strdup("Equinox Systems Inc");
		case ' ERG': return strdup("Ergo System");
		case ' ERI': return strdup("Ericsson Mobile Communications AB");
		case ' ERN': return strdup("Ericsson, Inc.");
		case ' ERP': return strdup("Euraplan GmbH");
		case ' ERS': return strdup("Eizo Rugged Solutions");
		case ' ERT': return strdup("Escort Insturments Corporation");
		case ' ESA': return strdup("Elbit Systems of America");
		case ' ESB': return strdup("Esterline Belgium BVBA");
		case ' ESC': return strdup("Eden Sistemas de Computacao S/A");
		case ' ESD': return strdup("Ensemble Designs, Inc");
		case ' ESG': return strdup("ELCON Systemtechnik GmbH");
		case ' ESI': return strdup("Extended Systems, Inc.");
		case ' ESK': return strdup("ES&S");
		case ' ESL': return strdup("Esterline Technologies");
		case ' ESN': return strdup("eSATURNUS");
		case ' ESS': return strdup("ESS Technology Inc");
		case ' EST': return strdup("Embedded Solution Technology");
		case ' ESY': return strdup("E-Systems Inc");
		case ' ETC': return strdup("Everton Technology Company Ltd");
		case ' ETD': return strdup("ELAN MICROELECTRONICS CORPORATION");
		case ' ETH': return strdup("Etherboot Project");
		case ' ETI': return strdup("Eclipse Tech Inc");
		case ' ETK': return strdup("eTEK Labs Inc.");
		case ' ETL': return strdup("Evertz Microsystems Ltd.");
		case ' ETS': return strdup("Electronic Trade Solutions Ltd");
		case ' ETT': return strdup("E-Tech Inc");
		case ' EUT': return strdup("Ericsson Mobile Networks B.V.");
		case ' EVE': return strdup("Advanced Micro Peripherals Ltd");
		case ' EVI': return strdup("eviateg GmbH");
		case ' EVX': return strdup("Everex");
		case ' EXA': return strdup("Exabyte");
		case ' EXC': return strdup("Excession Audio");
		case ' EXI': return strdup("Exide Electronics");
		case ' EXN': return strdup("RGB Systems, Inc. dba Extron Electronics");
		case ' EXP': return strdup("Data Export Corporation");
		case ' EXR': return strdup("Explorer Inc.");
		case ' EXT': return strdup("Exatech Computadores & Servicos Ltda");
		case ' EXX': return strdup("Exxact GmbH");
		case ' EXY': return strdup("Exterity Ltd");
		case ' EYE': return strdup("eyevis GmbH");
		case ' EYF': return strdup("eyefactive Gmbh");
		case ' EZE': return strdup("EzE Technologies");
		case ' EZP': return strdup("Storm Technology");
		case ' FAN': return strdup("Fantalooks Co., Ltd.");
		case ' FAR': return strdup("Farallon Computing");
		case ' FBI': return strdup("Interface Corporation");
		case ' FCB': return strdup("Furukawa Electric Company Ltd");
		case ' FCG': return strdup("First International Computer Ltd");
		case ' FCS': return strdup("Focus Enhancements, Inc.");
		case ' FDC': return strdup("Future Domain");
		case ' FDD': return strdup("Forth Dimension Displays Ltd");
		case ' FDI': return strdup("Future Designs, Inc.");
		case ' FDT': return strdup("Fujitsu Display Technologies Corp.");
		case ' FEC': return strdup("FURUNO ELECTRIC CO., LTD.");
		case ' FEL': return strdup("Fellowes & Questec");
		case ' FEN': return strdup("Fen Systems Ltd.");
		case ' FER': return strdup("Ferranti Int'L");
		case ' FFC': return strdup("FUJIFILM Corporation");
		case ' FFI': return strdup("Fairfield Industries");
		case ' FGD': return strdup("Lisa Draexlmaier GmbH");
		case ' FGL': return strdup("Fujitsu General Limited.");
		case ' FHL': return strdup("FHLP");
		case ' FIC': return strdup("Formosa Industrial Computing Inc");
		case ' FIL': return strdup("Forefront Int'l Ltd");
		case ' FIN': return strdup("Finecom Co., Ltd.");
		case ' FIR': return strdup("Chaplet Systems Inc");
		case ' FIS': return strdup("FLY-IT Simulators");
		case ' FIT': return strdup("Feature Integration Technology Inc.");
		case ' FJC': return strdup("Fujitsu Takamisawa Component Limited");
		case ' FJS': return strdup("Fujitsu Spain");
		case ' FJT': return strdup("F.J. Tieman BV");
		case ' FLE': return strdup("ADTI Media, Inc");
		case ' FLI': return strdup("Faroudja Laboratories");
		case ' FLY': return strdup("Butterfly Communications");
		case ' FMA': return strdup("Fast Multimedia AG");
		case ' FMC': return strdup("Ford Microelectronics Inc");
		case ' FMI': return strdup("Fellowes, Inc.");
		case ' FML': return strdup("Fujitsu Microelect Ltd");
		case ' FMZ': return strdup("Formoza-Altair");
		case ' FNC': return strdup("Fanuc LTD");
		case ' FNI': return strdup("Funai Electric Co., Ltd.");
		case ' FOA': return strdup("FOR-A Company Limited");
		case ' FOK': return strdup("Fokus Technologies GmbH");
		case ' FOS': return strdup("Foss Tecator");
		case ' FOV': return strdup("FOVE INC");
		case ' FOX': return strdup("HON HAI PRECISON IND.CO.,LTD.");
		case ' FPC': return strdup("Fingerprint Cards AB");
		case ' FPE': return strdup("Fujitsu Peripherals Ltd");
		case ' FPS': return strdup("Deltec Corporation");
		case ' FPX': return strdup("Cirel Systemes");
		case ' FRC': return strdup("Force Computers");
		case ' FRD': return strdup("Freedom Scientific BLV");
		case ' FRE': return strdup("Forvus Research Inc");
		case ' FRI': return strdup("Fibernet Research Inc");
		case ' FRO': return strdup("FARO Technologies");
		case ' FRS': return strdup("South Mountain Technologies, LTD");
		case ' FSC': return strdup("Future Systems Consulting KK");
		case ' FSI': return strdup("Fore Systems Inc");
		case ' FST': return strdup("Modesto PC Inc");
		case ' FTC': return strdup("Futuretouch Corporation");
		case ' FTE': return strdup("Frontline Test Equipment Inc.");
		case ' FTG': return strdup("FTG Data Systems");
		case ' FTI': return strdup("FastPoint Technologies, Inc.");
		case ' FTL': return strdup("FUJITSU TEN LIMITED");
		case ' FTN': return strdup("Fountain Technologies Inc");
		case ' FTR': return strdup("Mediasonic");
		case ' FTS': return strdup("FocalTech Systems Co., Ltd.");
		case ' FTW': return strdup("MindTribe Product Engineering, Inc.");
		case ' FUJ': return strdup("Fujitsu Ltd");
		case ' FUN': return strdup("sisel muhendislik");
		case ' FUS': return strdup("Fujitsu Siemens Computers GmbH");
		case ' FVC': return strdup("First Virtual Corporation");
		case ' FVX': return strdup("C-C-C Group Plc");
		case ' FWA': return strdup("Attero Tech, LLC");
		case ' FWR': return strdup("Flat Connections Inc");
		case ' FXX': return strdup("Fuji Xerox");
		case ' FZC': return strdup("Founder Group Shenzhen Co.");
		case ' FZI': return strdup("FZI Forschungszentrum Informatik");
		case ' GAC': return strdup("GreenArrays, Inc.");
		case ' GAG': return strdup("Gage Applied Sciences Inc");
		case ' GAL': return strdup("Galil Motion Control");
		case ' GAU': return strdup("Gaudi Co., Ltd.");
		case ' GCC': return strdup("GCC Technologies Inc");
		case ' GCI': return strdup("Gateway Comm. Inc");
		case ' GCS': return strdup("Grey Cell Systems Ltd");
		case ' GDC': return strdup("General Datacom");
		case ' GDI': return strdup("G. Diehl ISDN GmbH");
		case ' GDS': return strdup("GDS");
		case ' GDT': return strdup("Vortex Computersysteme GmbH");
		case ' GEC': return strdup("Gechic Corporation");
		case ' GED': return strdup("General Dynamics C4 Systems");
		case ' GEF': return strdup("GE Fanuc Embedded Systems");
		case ' GEH': return strdup("Abaco Systems, Inc.");
		case ' GEM': return strdup("Gem Plus");
		case ' GEN': return strdup("Genesys ATE Inc");
		case ' GEO': return strdup("GEO Sense");
		case ' GER': return strdup("GERMANEERS GmbH");
		case ' GES': return strdup("GES Singapore Pte Ltd");
		case ' GET': return strdup("Getac Technology Corporation");
		case ' GFM': return strdup("GFMesstechnik GmbH");
		case ' GFN': return strdup("Gefen Inc.");
		case ' GGL': return strdup("Google Inc.");
		case ' GIC': return strdup("General Inst. Corporation");
		case ' GIM': return strdup("Guillemont International");
		case ' GIP': return strdup("GI Provision Ltd");
		case ' GIS': return strdup("AT&T Global Info Solutions");
		case ' GJN': return strdup("Grand Junction Networks");
		case ' GLD': return strdup("Goldmund - Digital Audio SA");
		case ' GLE': return strdup("AD electronics");
		case ' GLM': return strdup("Genesys Logic");
		case ' GLS': return strdup("Gadget Labs LLC");
		case ' GMK': return strdup("GMK Electronic Design GmbH");
		case ' GML': return strdup("General Information Systems");
		case ' GMM': return strdup("GMM Research Inc");
		case ' GMN': return strdup("GEMINI 2000 Ltd");
		case ' GMX': return strdup("GMX Inc");
		case ' GND': return strdup("Gennum Corporation");
		case ' GNN': return strdup("GN Nettest Inc");
		case ' GNZ': return strdup("Gunze Ltd");
		case ' GOE': return strdup("GOEPEL electronic GmbH");
		case ' GPR': return strdup("GoPro, Inc.");
		case ' GRA': return strdup("Graphica Computer");
		case ' GRE': return strdup("GOLD RAIN ENTERPRISES CORP.");
		case ' GRH': return strdup("Granch Ltd");
		case ' GRM': return strdup("Garmin International");
		case ' GRV': return strdup("Advanced Gravis");
		case ' GRY': return strdup("Robert Gray Company");
		case ' GSB': return strdup("NIPPONDENCHI CO,.LTD");
		case ' GSC': return strdup("General Standards Corporation");
		case ' GSM': return strdup("Goldstar Company Ltd");
		case ' GSN': return strdup("Grandstream Networks, Inc.");
		case ' GST': return strdup("Graphic SystemTechnology");
		case ' GSY': return strdup("Grossenbacher Systeme AG");
		case ' GTC': return strdup("Graphtec Corporation");
		case ' GTI': return strdup("Goldtouch");
		case ' GTK': return strdup("G-Tech Corporation");
		case ' GTM': return strdup("Garnet System Company Ltd");
		case ' GTS': return strdup("Geotest Marvin Test Systems Inc");
		case ' GTT': return strdup("General Touch Technology Co., Ltd.");
		case ' GUD': return strdup("Guntermann & Drunck GmbH");
		case ' GUZ': return strdup("Guzik Technical Enterprises");
		case ' GVC': return strdup("GVC Corporation");
		case ' GVL': return strdup("Global Village Communication");
		case ' GWI': return strdup("GW Instruments");
		case ' GWK': return strdup("Gateworks Corporation");
		case ' GWY': return strdup("Gateway 2000");
		case ' GZE': return strdup("GUNZE Limited");
		case ' HAE': return strdup("Haider electronics");
		case ' HAI': return strdup("Haivision Systems Inc.");
		case ' HAL': return strdup("Halberthal");
		case ' HAN': return strdup("Hanchang System Corporation");
		case ' HAR': return strdup("Harris Corporation");
		case ' HAY': return strdup("Hayes Microcomputer Products Inc");
		case ' HCA': return strdup("DAT");
		case ' HCE': return strdup("Hitachi Consumer Electronics Co., Ltd");
		case ' HCL': return strdup("HCL America Inc");
		case ' HCM': return strdup("HCL Peripherals");
		case ' HCP': return strdup("Hitachi Computer Products Inc");
		case ' HCW': return strdup("Hauppauge Computer Works Inc");
		case ' HDC': return strdup("HardCom Elektronik & Datateknik");
		case ' HDI': return strdup("HD-INFO d.o.o.");
		case ' HDV': return strdup("Holografika kft.");
		case ' HEC': return strdup("Hisense Electric Co., Ltd.");
		case ' HEL': return strdup("Hitachi Micro Systems Europe Ltd");
		case ' HER': return strdup("Ascom Business Systems");
		case ' HET': return strdup("HETEC Datensysteme GmbH");
		case ' HHC': return strdup("HIRAKAWA HEWTECH CORP.");
		case ' HHI': return strdup("Fraunhofer Heinrich-Hertz-Institute");
		case ' HIB': return strdup("Hibino Corporation");
		case ' HIC': return strdup("Hitachi Information Technology Co., Ltd.");
		case ' HII': return strdup("Harman International Industries, Inc");
		case ' HIK': return strdup("Hikom Co., Ltd.");
		case ' HIL': return strdup("Hilevel Technology");
		case ' HIQ': return strdup("Kaohsiung Opto Electronics Americas, Inc.");
		case ' HIS': return strdup("Hope Industrial Systems, Inc.");
		case ' HIT': return strdup("Hitachi America Ltd");
		case ' HJI': return strdup("Harris & Jeffries Inc");
		case ' HKA': return strdup("HONKO MFG. CO., LTD.");
		case ' HKC': return strdup("HKC OVERSEAS LIMITED");
		case ' HKG': return strdup("Josef Heim KG");
		case ' HLG': return strdup("China Hualu Group Co., Ltd.");
		case ' HMC': return strdup("Hualon Microelectric Corporation");
		case ' HMK': return strdup("hmk Daten-System-Technik BmbH");
		case ' HMX': return strdup("HUMAX Co., Ltd.");
		case ' HNS': return strdup("Hughes Network Systems");
		case ' HOB': return strdup("HOB Electronic GmbH");
		case ' HOE': return strdup("Hosiden Corporation");
		case ' HOL': return strdup("Holoeye Photonics AG");
		case ' HON': return strdup("Sonitronix");
		case ' HPA': return strdup("Zytor Communications");
		case ' HPC': return strdup("Hewlett-Packard Co.");
		case ' HPD': return strdup("Hewlett Packard");
		case ' HPE': return strdup("Hewlett Packard Enterprise");
		case ' HPI': return strdup("Headplay, Inc.");
		case ' HPK': return strdup("HAMAMATSU PHOTONICS K.K.");
		case ' HPN': return strdup("HP Inc.");
		case ' HPQ': return strdup("Hewlett-Packard Co.");
		case ' HPR': return strdup("H.P.R. Electronics GmbH");
		case ' HRC': return strdup("Hercules");
		case ' HRE': return strdup("Qingdao Haier Electronics Co., Ltd.");
		case ' HRI': return strdup("Hall Research");
		case ' HRL': return strdup("Herolab GmbH");
		case ' HRS': return strdup("Harris Semiconductor");
		case ' HRT': return strdup("HERCULES");
		case ' HSC': return strdup("Hagiwara Sys-Com Company Ltd");
		case ' HSD': return strdup("HannStar Display Corp");
		case ' HSM': return strdup("AT&T Microelectronics");
		case ' HSP': return strdup("HannStar Display Corp");
		case ' HST': return strdup("Horsent Technology Co., Ltd.");
		case ' HTC': return strdup("Hitachi Ltd");
		case ' HTI': return strdup("Hampshire Company, Inc.");
		case ' HTK': return strdup("Holtek Microelectronics Inc");
		case ' HTL': return strdup("HTBLuVA Mödling");
		case ' HTR': return strdup("Shenzhen ZhuoYi HengTong Computer Technology Limited");
		case ' HTX': return strdup("Hitex Systementwicklung GmbH");
		case ' HUB': return strdup("GAI-Tronics, A Hubbell Company");
		case ' HUK': return strdup("Hoffmann + Krippner GmbH");
		case ' HUM': return strdup("IMP Electronics Ltd.");
		case ' HVR': return strdup("HTC Corportation");
		case ' HWA': return strdup("Harris Canada Inc");
		case ' HWC': return strdup("DBA Hans Wedemeyer");
		case ' HWD': return strdup("Highwater Designs Ltd");
		case ' HWP': return strdup("Hewlett Packard");
		case ' HXM': return strdup("Hexium Ltd.");
		case ' HYC': return strdup("Hypercope Gmbh Aachen");
		case ' HYD': return strdup("Hydis Technologies.Co.,LTD");
		case ' HYO': return strdup("HYC CO., LTD.");
		case ' HYP': return strdup("Hyphen Ltd");
		case ' HYR': return strdup("Hypertec Pty Ltd");
		case ' HYT': return strdup("Heng Yu Technology (HK) Limited");
		case ' HYV': return strdup("Hynix Semiconductor");
		case ' IAD': return strdup("IAdea Corporation");
		case ' IAF': return strdup("Institut f r angewandte Funksystemtechnik GmbH");
		case ' IAI': return strdup("Integration Associates, Inc.");
		case ' IAT': return strdup("IAT Germany GmbH");
		case ' IBC': return strdup("Integrated Business Systems");
		case ' IBI': return strdup("INBINE.CO.LTD");
		case ' IBM': return strdup("IBM Brasil");
		case ' IBP': return strdup("IBP Instruments GmbH");
		case ' IBR': return strdup("IBR GmbH");
		case ' ICA': return strdup("ICA Inc");
		case ' ICC': return strdup("BICC Data Networks Ltd");
		case ' ICD': return strdup("ICD Inc");
		case ' ICE': return strdup("IC Ensemble");
		case ' ICI': return strdup("Infotek Communication Inc");
		case ' ICM': return strdup("Intracom SA");
		case ' ICN': return strdup("Sanyo Icon");
		case ' ICO': return strdup("Intel Corp");
		case ' ICP': return strdup("ICP Electronics, Inc./iEi Technology Corp.");
		case ' ICS': return strdup("Integrated Circuit Systems");
		case ' ICV': return strdup("Inside Contactless");
		case ' ICX': return strdup("ICCC A/S");
		case ' IDC': return strdup("International Datacasting Corporation");
		case ' IDE': return strdup("IDE Associates");
		case ' IDK': return strdup("IDK Corporation");
		case ' IDN': return strdup("Idneo Technologies");
		case ' IDO': return strdup("IDEO Product Development");
		case ' IDP': return strdup("Integrated Device Technology, Inc.");
		case ' IDS': return strdup("Interdigital Sistemas de Informacao");
		case ' IDT': return strdup("International Display Technology");
		case ' IDX': return strdup("IDEXX Labs");
		case ' IEC': return strdup("Interlace Engineering Corporation");
		case ' IEE': return strdup("IEE");
		case ' IEI': return strdup("Interlink Electronics");
		case ' IFS': return strdup("In Focus Systems Inc");
		case ' IFT': return strdup("Informtech");
		case ' IFX': return strdup("Infineon Technologies AG");
		case ' IFZ': return strdup("Infinite Z");
		case ' IGC': return strdup("Intergate Pty Ltd");
		case ' IGM': return strdup("IGM Communi");
		case ' IHE': return strdup("InHand Electronics");
		case ' IIC': return strdup("ISIC Innoscan Industrial Computers A/S");
		case ' III': return strdup("Intelligent Instrumentation");
		case ' IIN': return strdup("IINFRA Co., Ltd");
		case ' IIT': return strdup("Informatik Information Technologies");
		case ' IKE': return strdup("Ikegami Tsushinki Co. Ltd.");
		case ' IKS': return strdup("Ikos Systems Inc");
		case ' ILC': return strdup("Image Logic Corporation");
		case ' ILS': return strdup("Innotech Corporation");
		case ' IMA': return strdup("Imagraph");
		case ' IMB': return strdup("ART s.r.l.");
		case ' IMC': return strdup("IMC Networks");
		case ' IMD': return strdup("ImasDe Canarias S.A.");
		case ' IME': return strdup("Imagraph");
		case ' IMG': return strdup("IMAGENICS Co., Ltd.");
		case ' IMI': return strdup("International Microsystems Inc");
		case ' IMM': return strdup("Immersion Corporation");
		case ' IMN': return strdup("Impossible Production");
		case ' IMP': return strdup("Impinj");
		case ' IMT': return strdup("Inmax Technology Corporation");
		case ' INA': return strdup("Inventec Corporation");
		case ' INC': return strdup("Home Row Inc");
		case ' IND': return strdup("ILC");
		case ' INE': return strdup("Inventec Electronics (M) Sdn. Bhd.");
		case ' INF': return strdup("Inframetrics Inc");
		case ' ING': return strdup("Integraph Corporation");
		case ' INI': return strdup("Initio Corporation");
		case ' INK': return strdup("Indtek Co., Ltd.");
		case ' INL': return strdup("InnoLux Display Corporation");
		case ' INM': return strdup("InnoMedia Inc");
		case ' INN': return strdup("Innovent Systems, Inc.");
		case ' INO': return strdup("Innolab Pte Ltd");
		case ' INP': return strdup("Interphase Corporation");
		case ' INS': return strdup("Ines GmbH");
		case ' INT': return strdup("Interphase Corporation");
		case ' INU': return strdup("Inovatec S.p.A.");
		case ' INV': return strdup("Inviso, Inc.");
		case ' INX': return strdup("Communications Supply Corporation (A division of WESCO)");
		case ' INZ': return strdup("Best Buy");
		case ' IOA': return strdup("CRE Technology Corporation");
		case ' IOD': return strdup("I-O Data Device Inc");
		case ' IOM': return strdup("Iomega");
		case ' ION': return strdup("Inside Out Networks");
		case ' IOS': return strdup("i-O Display System");
		case ' IOT': return strdup("I/OTech Inc");
		case ' IPC': return strdup("IPC Corporation");
		case ' IPD': return strdup("Industrial Products Design, Inc.");
		case ' IPI': return strdup("Intelligent Platform Management Interface (IPMI) forum (Intel, HP, NEC, Dell)");
		case ' IPM': return strdup("IPM Industria Politecnica Meridionale SpA");
		case ' IPN': return strdup("Performance Technologies");
		case ' IPP': return strdup("IP Power Technologies GmbH");
		case ' IPQ': return strdup("IP3 Technology Ltd.");
		case ' IPR': return strdup("Ithaca Peripherals");
		case ' IPS': return strdup("IPS, Inc. (Intellectual Property Solutions, Inc.)");
		case ' IPT': return strdup("International Power Technologies");
		case ' IPW': return strdup("IPWireless, Inc");
		case ' IQI': return strdup("IneoQuest Technologies, Inc");
		case ' IQT': return strdup("IMAGEQUEST Co., Ltd");
		case ' IRD': return strdup("Irdata");
		case ' ISA': return strdup("Symbol Technologies");
		case ' ISC': return strdup("Id3 Semiconductors");
		case ' ISG': return strdup("Insignia Solutions Inc");
		case ' ISI': return strdup("Interface Solutions");
		case ' ISL': return strdup("Isolation Systems");
		case ' ISM': return strdup("Image Stream Medical");
		case ' ISP': return strdup("IntreSource Systems Pte Ltd");
		case ' ISR': return strdup("INSIS Co., LTD.");
		case ' ISS': return strdup("ISS Inc");
		case ' IST': return strdup("Intersolve Technologies");
		case ' ISY': return strdup("International Integrated Systems,Inc.(IISI)");
		case ' ITA': return strdup("Itausa Export North America");
		case ' ITC': return strdup("Intercom Inc");
		case ' ITD': return strdup("Internet Technology Corporation");
		case ' ITE': return strdup("Integrated Tech Express Inc");
		case ' ITI': return strdup("VanErum Group");
		case ' ITK': return strdup("ITK Telekommunikation AG");
		case ' ITL': return strdup("Inter-Tel");
		case ' ITM': return strdup("ITM inc.");
		case ' ITN': return strdup("The NTI Group");
		case ' ITP': return strdup("IT-PRO Consulting und Systemhaus GmbH");
		case ' ITR': return strdup("Infotronic America, Inc.");
		case ' ITS': return strdup("IDTECH");
		case ' ITT': return strdup("I&T Telecom.");
		case ' ITX': return strdup("integrated Technology Express Inc");
		case ' IUC': return strdup("ICSL");
		case ' IVI': return strdup("Intervoice Inc");
		case ' IVM': return strdup("Iiyama North America");
		case ' IVS': return strdup("Intevac Photonics Inc.");
		case ' IWR': return strdup("Icuiti Corporation");
		case ' IWX': return strdup("Intelliworxx, Inc.");
		case ' IXD': return strdup("Intertex Data AB");
		case ' IXN': return strdup("Shenzhen Inet Mobile Internet Technology Co., LTD");
		case ' JAC': return strdup("Astec Inc");
		case ' JAE': return strdup("Japan Aviation Electronics Industry, Limited");
		case ' JAS': return strdup("Janz Automationssysteme AG");
		case ' JAT': return strdup("Jaton Corporation");
		case ' JAZ': return strdup("Carrera Computer Inc");
		case ' JCE': return strdup("Jace Tech Inc");
		case ' JDI': return strdup("Japan Display Inc.");
		case ' JDL': return strdup("Japan Digital Laboratory Co.,Ltd.");
		case ' JEN': return strdup("N-Vision");
		case ' JET': return strdup("JET POWER TECHNOLOGY CO., LTD.");
		case ' JFX': return strdup("Jones Futurex Inc");
		case ' JGD': return strdup("University College");
		case ' JIC': return strdup("Jaeik Information & Communication Co., Ltd.");
		case ' JKC': return strdup("JVC KENWOOD Corporation");
		case ' JMT': return strdup("Micro Technical Company Ltd");
		case ' JPC': return strdup("JPC Technology Limited");
		case ' JPW': return strdup("Wallis Hamilton Industries");
		case ' JQE': return strdup("CNet Technical Inc");
		case ' JSD': return strdup("JS DigiTech, Inc");
		case ' JSI': return strdup("Jupiter Systems, Inc.");
		case ' JSK': return strdup("SANKEN ELECTRIC CO., LTD");
		case ' JTS': return strdup("JS Motorsports");
		case ' JTY': return strdup("jetway security micro,inc");
		case ' JUK': return strdup("Janich & Klass Computertechnik GmbH");
		case ' JUP': return strdup("Jupiter Systems");
		case ' JVC': return strdup("JVC");
		case ' JWD': return strdup("Video International Inc.");
		case ' JWL': return strdup("Jewell Instruments, LLC");
		case ' JWS': return strdup("JWSpencer & Co.");
		case ' JWY': return strdup("Jetway Information Co., Ltd");
		case ' KAR': return strdup("Karna");
		case ' KBI': return strdup("Kidboard Inc");
		case ' KBL': return strdup("Kobil Systems GmbH");
		case ' KCD': return strdup("Chunichi Denshi Co.,LTD.");
		case ' KCL': return strdup("Keycorp Ltd");
		case ' KDE': return strdup("KDE");
		case ' KDK': return strdup("Kodiak Tech");
		case ' KDM': return strdup("Korea Data Systems Co., Ltd.");
		case ' KDS': return strdup("KDS USA");
		case ' KDT': return strdup("KDDI Technology Corporation");
		case ' KEC': return strdup("Kyushu Electronics Systems Inc");
		case ' KEM': return strdup("Kontron Embedded Modules GmbH");
		case ' KES': return strdup("Kesa Corporation");
		case ' KEU': return strdup("Kontron Europe GmbH");
		case ' KEY': return strdup("Key Tech Inc");
		case ' KFC': return strdup("SCD Tech");
		case ' KFE': return strdup("Komatsu Forest");
		case ' KFX': return strdup("Kofax Image Products");
		case ' KGI': return strdup("Klipsch Group, Inc");
		case ' KGL': return strdup("KEISOKU GIKEN Co.,Ltd.");
		case ' KIO': return strdup("Kionix, Inc.");
		case ' KIS': return strdup("KiSS Technology A/S");
		case ' KMC': return strdup("Mitsumi Company Ltd");
		case ' KME': return strdup("KIMIN Electronics Co., Ltd.");
		case ' KML': return strdup("Kensington Microware Ltd");
		case ' KMR': return strdup("Kramer Electronics Ltd. International");
		case ' KNC': return strdup("Konica corporation");
		case ' KNX': return strdup("Nutech Marketing PTL");
		case ' KOB': return strdup("Kobil Systems GmbH");
		case ' KOD': return strdup("Eastman Kodak Company");
		case ' KOE': return strdup("KOLTER ELECTRONIC");
		case ' KOL': return strdup("Kollmorgen Motion Technologies Group");
		case ' KOU': return strdup("KOUZIRO Co.,Ltd.");
		case ' KOW': return strdup("KOWA Company,LTD.");
		case ' KPC': return strdup("King Phoenix Company");
		case ' KPT': return strdup("TPK Holding Co., Ltd");
		case ' KRL': return strdup("Krell Industries Inc.");
		case ' KRM': return strdup("Kroma Telecom");
		case ' KRY': return strdup("Kroy LLC");
		case ' KSC': return strdup("Kinetic Systems Corporation");
		case ' KSG': return strdup("KUPA China Shenzhen Micro Technology Co., Ltd. Gold Institute");
		case ' KSL': return strdup("Karn Solutions Ltd.");
		case ' KSX': return strdup("King Tester Corporation");
		case ' KTC': return strdup("Kingston Tech Corporation");
		case ' KTD': return strdup("Takahata Electronics Co.,Ltd.");
		case ' KTE': return strdup("K-Tech");
		case ' KTG': return strdup("Kayser-Threde GmbH");
		case ' KTI': return strdup("Konica Technical Inc");
		case ' KTK': return strdup("Key Tronic Corporation");
		case ' KTN': return strdup("Katron Tech Inc");
		case ' KUR': return strdup("Kurta Corporation");
		case ' KVA': return strdup("Kvaser AB");
		case ' KVX': return strdup("KeyView");
		case ' KWD': return strdup("Kenwood Corporation");
		case ' KYC': return strdup("Kyocera Corporation");
		case ' KYE': return strdup("KYE Syst Corporation");
		case ' KYK': return strdup("Samsung Electronics America Inc");
		case ' KYN': return strdup("KEYENCE CORPORATION");
		case ' KZI': return strdup("K-Zone International co. Ltd.");
		case ' KZN': return strdup("K-Zone International");
		case ' LAB': return strdup("ACT Labs Ltd");
		case ' LAC': return strdup("LaCie");
		case ' LAF': return strdup("Microline");
		case ' LAG': return strdup("Laguna Systems");
		case ' LAN': return strdup("Sodeman Lancom Inc");
		case ' LAS': return strdup("LASAT Comm. A/S");
		case ' LAV': return strdup("Lava Computer MFG Inc");
		case ' LBO': return strdup("Lubosoft");
		case ' LCC': return strdup("LCI");
		case ' LCD': return strdup("Toshiba Matsushita Display Technology Co., Ltd");
		case ' LCE': return strdup("La Commande Electronique");
		case ' LCI': return strdup("Lite-On Communication Inc");
		case ' LCM': return strdup("Latitude Comm.");
		case ' LCN': return strdup("LEXICON");
		case ' LCS': return strdup("Longshine Electronics Company");
		case ' LCT': return strdup("Labcal Technologies");
		case ' LDN': return strdup("Laserdyne Technologies");
		case ' LDT': return strdup("LogiDataTech Electronic GmbH");
		case ' LEC': return strdup("Lectron Company Ltd");
		case ' LED': return strdup("Long Engineering Design Inc");
		case ' LEG': return strdup("Legerity, Inc");
		case ' LEN': return strdup("Lenovo Group Limited");
		case ' LEO': return strdup("First International Computer Inc");
		case ' LEX': return strdup("Lexical Ltd");
		case ' LGC': return strdup("Logic Ltd");
		case ' LGI': return strdup("Logitech Inc");
		case ' LGS': return strdup("LG Semicom Company Ltd");
		case ' LGX': return strdup("Lasergraphics, Inc.");
		case ' LHA': return strdup("Lars Haagh ApS");
		case ' LHE': return strdup("Lung Hwa Electronics Company Ltd");
		case ' LHT': return strdup("Lighthouse Technologies Limited");
		case ' LIN': return strdup("Lenovo Beijing Co. Ltd.");
		case ' LIP': return strdup("Linked IP GmbH");
		case ' LIT': return strdup("Lithics Silicon Technology");
		case ' LJX': return strdup("Datalogic Corporation");
		case ' LKM': return strdup("Likom Technology Sdn. Bhd.");
		case ' LLL': return strdup("L-3 Communications");
		case ' LMG': return strdup("Lucent Technologies");
		case ' LMI': return strdup("Lexmark Int'l Inc");
		case ' LMP': return strdup("Leda Media Products");
		case ' LMT': return strdup("Laser Master");
		case ' LND': return strdup("Land Computer Company Ltd");
		case ' LNK': return strdup("Link Tech Inc");
		case ' LNR': return strdup("Linear Systems Ltd.");
		case ' LNT': return strdup("LANETCO International");
		case ' LNV': return strdup("Lenovo");
		case ' LNX': return strdup("The Linux Foundation");
		case ' LOC': return strdup("Locamation B.V.");
		case ' LOE': return strdup("Loewe Opta GmbH");
		case ' LOG': return strdup("Logicode Technology Inc");
		case ' LOL': return strdup("Litelogic Operations Ltd");
		case ' LPE': return strdup("El-PUSK Co., Ltd.");
		case ' LPI': return strdup("Design Technology");
		case ' LPL': return strdup("DO NOT USE - LPL");
		case ' LSC': return strdup("LifeSize Communications");
		case ' LSD': return strdup("Intersil Corporation");
		case ' LSI': return strdup("Loughborough Sound Images");
		case ' LSJ': return strdup("LSI Japan Company Ltd");
		case ' LSL': return strdup("Logical Solutions");
		case ' LSY': return strdup("LSI Systems Inc");
		case ' LTC': return strdup("Labtec Inc");
		case ' LTI': return strdup("Jongshine Tech Inc");
		case ' LTK': return strdup("Lucidity Technology Company Ltd");
		case ' LTN': return strdup("Litronic Inc");
		case ' LTS': return strdup("LTS Scale LLC");
		case ' LTV': return strdup("Leitch Technology International Inc.");
		case ' LTW': return strdup("Lightware, Inc");
		case ' LUC': return strdup("Lucent Technologies");
		case ' LUM': return strdup("Lumagen, Inc.");
		case ' LUX': return strdup("Luxxell Research Inc");
		case ' LVI': return strdup("LVI Low Vision International AB");
		case ' LWC': return strdup("Labway Corporation");
		case ' LWR': return strdup("Lightware Visual Engineering");
		case ' LWW': return strdup("Lanier Worldwide");
		case ' LXC': return strdup("LXCO Technologies AG");
		case ' LXN': return strdup("Luxeon");
		case ' LXS': return strdup("ELEA CardWare");
		case ' LZX': return strdup("Lightwell Company Ltd");
		case ' MAC': return strdup("MAC System Company Ltd");
		case ' MAD': return strdup("Xedia Corporation");
		case ' MAE': return strdup("Maestro Pty Ltd");
		case ' MAG': return strdup("MAG InnoVision");
		case ' MAI': return strdup("Mutoh America Inc");
		case ' MAL': return strdup("Meridian Audio Ltd");
		case ' MAN': return strdup("LGIC");
		case ' MAS': return strdup("Mass Inc.");
		case ' MAT': return strdup("Matsushita Electric Ind. Company Ltd");
		case ' MAX': return strdup("Rogen Tech Distribution Inc");
		case ' MAY': return strdup("Maynard Electronics");
		case ' MAZ': return strdup("MAZeT GmbH");
		case ' MBC': return strdup("MBC");
		case ' MBD': return strdup("Microbus PLC");
		case ' MBM': return strdup("Marshall Electronics");
		case ' MBV': return strdup("Moreton Bay");
		case ' MCA': return strdup("American Nuclear Systems Inc");
		case ' MCC': return strdup("Micro Industries");
		case ' MCD': return strdup("McDATA Corporation");
		case ' MCE': return strdup("Metz-Werke GmbH & Co KG");
		case ' MCG': return strdup("Motorola Computer Group");
		case ' MCI': return strdup("Micronics Computers");
		case ' MCL': return strdup("Motorola Communications Israel");
		case ' MCM': return strdup("Metricom Inc");
		case ' MCN': return strdup("Micron Electronics Inc");
		case ' MCO': return strdup("Motion Computing Inc.");
		case ' MCP': return strdup("Magni Systems Inc");
		case ' MCQ': return strdup("Mat's Computers");
		case ' MCR': return strdup("Marina Communicaitons");
		case ' MCS': return strdup("Micro Computer Systems");
		case ' MCT': return strdup("Microtec");
		case ' MCX': return strdup("Millson Custom Solutions Inc.");
		case ' MDA': return strdup("Media4 Inc");
		case ' MDC': return strdup("Midori Electronics");
		case ' MDD': return strdup("MODIS");
		case ' MDF': return strdup("MILDEF AB");
		case ' MDG': return strdup("Madge Networks");
		case ' MDI': return strdup("Micro Design Inc");
		case ' MDK': return strdup("Mediatek Corporation");
		case ' MDO': return strdup("Panasonic");
		case ' MDR': return strdup("Medar Inc");
		case ' MDS': return strdup("Micro Display Systems Inc");
		case ' MDT': return strdup("Magus Data Tech");
		case ' MDV': return strdup("MET Development Inc");
		case ' MDX': return strdup("MicroDatec GmbH");
		case ' MDY': return strdup("Microdyne Inc");
		case ' MEC': return strdup("Mega System Technologies Inc");
		case ' MED': return strdup("Messeltronik Dresden GmbH");
		case ' MEE': return strdup("Mitsubishi Electric Engineering Co., Ltd.");
		case ' MEG': return strdup("Abeam Tech Ltd.");
		case ' MEI': return strdup("Panasonic Industry Company");
		case ' MEJ': return strdup("Mac-Eight Co., LTD.");
		case ' MEK': return strdup("Mediaedge Corporation");
		case ' MEL': return strdup("Mitsubishi Electric Corporation");
		case ' MEN': return strdup("MEN Mikroelectronik Nueruberg GmbH");
		case ' MEP': return strdup("Meld Technology");
		case ' MEQ': return strdup("Matelect Ltd.");
		case ' MET': return strdup("Metheus Corporation");
		case ' MEU': return strdup("MPL AG, Elektronik-Unternehmen");
		case ' MEX': return strdup("MSC Vertriebs GmbH");
		case ' MFG': return strdup("MicroField Graphics Inc");
		case ' MFI': return strdup("Micro Firmware");
		case ' MFR': return strdup("MediaFire Corp.");
		case ' MGA': return strdup("Mega System Technologies, Inc.");
		case ' MGC': return strdup("Mentor Graphics Corporation");
		case ' MGE': return strdup("Schneider Electric S.A.");
		case ' MGL': return strdup("M-G Technology Ltd");
		case ' MGT': return strdup("Megatech R & D Company");
		case ' MIC': return strdup("Micom Communications Inc");
		case ' MID': return strdup("miro Displays");
		case ' MII': return strdup("Mitec Inc");
		case ' MIL': return strdup("Marconi Instruments Ltd");
		case ' MIM': return strdup("Mimio – A Newell Rubbermaid Company");
		case ' MIN': return strdup("Minicom Digital Signage");
		case ' MIP': return strdup("micronpc.com");
		case ' MIR': return strdup("Miro Computer Prod.");
		case ' MIS': return strdup("Modular Industrial Solutions Inc");
		case ' MIT': return strdup("MCM Industrial Technology GmbH");
		case ' MIV': return strdup("MicroImage Video Systems");
		case ' MJI': return strdup("MARANTZ JAPAN, INC.");
		case ' MJS': return strdup("MJS Designs");
		case ' MKC': return strdup("Media Tek Inc.");
		case ' MKS': return strdup("MK Seiko Co., Ltd.");
		case ' MKT': return strdup("MICROTEK Inc.");
		case ' MKV': return strdup("Trtheim Technology");
		case ' MLD': return strdup("Deep Video Imaging Ltd");
		case ' MLG': return strdup("Micrologica AG");
		case ' MLI': return strdup("McIntosh Laboratory Inc.");
		case ' MLL': return strdup("Millogic Ltd.");
		case ' MLM': return strdup("Millennium Engineering Inc");
		case ' MLN': return strdup("Mark Levinson");
		case ' MLP': return strdup("Magic Leap");
		case ' MLS': return strdup("Milestone EPE");
		case ' MLT': return strdup("Wanlida Group Co., Ltd.");
		case ' MLX': return strdup("Mylex Corporation");
		case ' MMA': return strdup("Micromedia AG");
		case ' MMD': return strdup("Micromed Biotecnologia Ltd");
		case ' MMF': return strdup("Minnesota Mining and Manufacturing");
		case ' MMI': return strdup("Multimax");
		case ' MMM': return strdup("Electronic Measurements");
		case ' MMN': return strdup("MiniMan Inc");
		case ' MMS': return strdup("MMS Electronics");
		case ' MNC': return strdup("Mini Micro Methods Ltd");
		case ' MNI': return strdup("Marseille, Inc.");
		case ' MNL': return strdup("Monorail Inc");
		case ' MNP': return strdup("Microcom");
		case ' MOD': return strdup("Modular Technology");
		case ' MOM': return strdup("Momentum Data Systems");
		case ' MOS': return strdup("Moses Corporation");
		case ' MOT': return strdup("Motorola UDS");
		case ' MPC': return strdup("M-Pact Inc");
		case ' MPI': return strdup("Mediatrix Peripherals Inc");
		case ' MPJ': return strdup("Microlab");
		case ' MPL': return strdup("Maple Research Inst. Company Ltd");
		case ' MPN': return strdup("Mainpine Limited");
		case ' MPS': return strdup("mps Software GmbH");
		case ' MPX': return strdup("Micropix Technologies, Ltd.");
		case ' MQP': return strdup("MultiQ Products AB");
		case ' MRA': return strdup("Miranda Technologies Inc");
		case ' MRC': return strdup("Marconi Simulation & Ty-Coch Way Training");
		case ' MRD': return strdup("MicroDisplay Corporation");
		case ' MRK': return strdup("Maruko & Company Ltd");
		case ' MRL': return strdup("Miratel");
		case ' MRO': return strdup("Medikro Oy");
		case ' MRT': return strdup("Merging Technologies");
		case ' MSA': return strdup("Micro Systemation AB");
		case ' MSC': return strdup("Mouse Systems Corporation");
		case ' MSD': return strdup("Datenerfassungs- und Informationssysteme");
		case ' MSF': return strdup("M-Systems Flash Disk Pioneers");
		case ' MSG': return strdup("MSI GmbH");
		case ' MSH': return strdup("Microsoft");
		case ' MSI': return strdup("Microstep");
		case ' MSK': return strdup("Megasoft Inc");
		case ' MSL': return strdup("MicroSlate Inc.");
		case ' MSM': return strdup("Advanced Digital Systems");
		case ' MSP': return strdup("Mistral Solutions [P] Ltd.");
		case ' MSR': return strdup("MASPRO DENKOH Corp.");
		case ' MST': return strdup("MS Telematica");
		case ' MSU': return strdup("motorola");
		case ' MSV': return strdup("Mosgi Corporation");
		case ' MSX': return strdup("Micomsoft Co., Ltd.");
		case ' MSY': return strdup("MicroTouch Systems Inc");
		case ' MTA': return strdup("Meta Watch Ltd");
		case ' MTB': return strdup("Media Technologies Ltd.");
		case ' MTC': return strdup("Mars-Tech Corporation");
		case ' MTD': return strdup("MindTech Display Co. Ltd");
		case ' MTE': return strdup("MediaTec GmbH");
		case ' MTH': return strdup("Micro-Tech Hearing Instruments");
		case ' MTI': return strdup("MaxCom Technical Inc");
		case ' MTJ': return strdup("MicroTechnica Co.,Ltd.");
		case ' MTK': return strdup("Microtek International Inc.");
		case ' MTL': return strdup("Mitel Corporation");
		case ' MTM': return strdup("Motium");
		case ' MTN': return strdup("Mtron Storage Technology Co., Ltd.");
		case ' MTR': return strdup("Mitron computer Inc");
		case ' MTS': return strdup("Multi-Tech Systems");
		case ' MTU': return strdup("Mark of the Unicorn Inc");
		case ' MTX': return strdup("Matrox");
		case ' MUD': return strdup("Multi-Dimension Institute");
		case ' MUK': return strdup("Mainpine Limited");
		case ' MVD': return strdup("Microvitec PLC");
		case ' MVI': return strdup("Media Vision Inc");
		case ' MVM': return strdup("SOBO VISION");
		case ' MVN': return strdup("Meta Company");
		case ' MVS': return strdup("Microvision");
		case ' MVX': return strdup("COM 1");
		case ' MWI': return strdup("Multiwave Innovation Pte Ltd");
		case ' MWR': return strdup("mware");
		case ' MWY': return strdup("Microway Inc");
		case ' MXD': return strdup("MaxData Computer GmbH & Co.KG");
		case ' MXI': return strdup("Macronix Inc");
		case ' MXL': return strdup("Hitachi Maxell, Ltd.");
		case ' MXP': return strdup("Maxpeed Corporation");
		case ' MXT': return strdup("Maxtech Corporation");
		case ' MXV': return strdup("MaxVision Corporation");
		case ' MYA': return strdup("Monydata");
		case ' MYR': return strdup("Myriad Solutions Ltd");
		case ' MYX': return strdup("Micronyx Inc");
		case ' NAC': return strdup("Ncast Corporation");
		case ' NAD': return strdup("NAD Electronics");
		case ' NAK': return strdup("Nakano Engineering Co.,Ltd.");
		case ' NAL': return strdup("Network Alchemy");
		case ' NAT': return strdup("NaturalPoint Inc.");
		case ' NAV': return strdup("Navigation Corporation");
		case ' NAX': return strdup("Naxos Tecnologia");
		case ' NBL': return strdup("N*Able Technologies Inc");
		case ' NBS': return strdup("National Key Lab. on ISN");
		case ' NBT': return strdup("NingBo Bestwinning Technology CO., Ltd");
		case ' NCA': return strdup("Nixdorf Company");
		case ' NCC': return strdup("NCR Corporation");
		case ' NCE': return strdup("Norcent Technology, Inc.");
		case ' NCI': return strdup("NewCom Inc");
		case ' NCL': return strdup("NetComm Ltd");
		case ' NCP': return strdup("Najing CEC Panda FPD Technology CO. ltd");
		case ' NCR': return strdup("NCR Electronics");
		case ' NCS': return strdup("Northgate Computer Systems");
		case ' NCT': return strdup("NEC CustomTechnica, Ltd.");
		case ' NDC': return strdup("National DataComm Corporaiton");
		case ' NDF': return strdup("NDF Special Light Products B.V.");
		case ' NDI': return strdup("National Display Systems");
		case ' NDK': return strdup("Naitoh Densei CO., LTD.");
		case ' NDL': return strdup("Network Designers");
		case ' NDS': return strdup("Nokia Data");
		case ' NEC': return strdup("NEC Corporation");
		case ' NEO': return strdup("NEO TELECOM CO.,LTD.");
		case ' NES': return strdup("INNES");
		case ' NET': return strdup("Mettler Toledo");
		case ' NEU': return strdup("NEUROTEC - EMPRESA DE PESQUISA E DESENVOLVIMENTO EM BIOMEDICINA");
		case ' NEX': return strdup("Nexgen Mediatech Inc.,");
		case ' NFC': return strdup("BTC Korea Co., Ltd");
		case ' NFS': return strdup("Number Five Software");
		case ' NGC': return strdup("Network General");
		case ' NGS': return strdup("A D S Exports");
		case ' NHT': return strdup("Vinci Labs");
		case ' NIC': return strdup("National Instruments Corporation");
		case ' NIS': return strdup("Nissei Electric Company");
		case ' NIT': return strdup("Network Info Technology");
		case ' NIX': return strdup("Seanix Technology Inc");
		case ' NLC': return strdup("Next Level Communications");
		case ' NME': return strdup("Navico, Inc.");
		case ' NMP': return strdup("Nokia Mobile Phones");
		case ' NMS': return strdup("Natural Micro System");
		case ' NMV': return strdup("NEC-Mitsubishi Electric Visual Systems Corporation");
		case ' NMX': return strdup("Neomagic");
		case ' NNC': return strdup("NNC");
		case ' NOD': return strdup("3NOD Digital Technology Co. Ltd.");
		case ' NOE': return strdup("NordicEye AB");
		case ' NOI': return strdup("North Invent A/S");
		case ' NOK': return strdup("Nokia Display Products");
		case ' NOR': return strdup("Norand Corporation");
		case ' NOT': return strdup("Not Limited Inc");
		case ' NPA': return strdup("Arvanics");
		case ' NPI': return strdup("Network Peripherals Inc");
		case ' NRL': return strdup("U.S. Naval Research Lab");
		case ' NRT': return strdup("Beijing Northern Radiantelecom Co.");
		case ' NRV': return strdup("Taugagreining hf");
		case ' NSA': return strdup("NeuroSky, Inc.");
		case ' NSC': return strdup("National Semiconductor Corporation");
		case ' NSI': return strdup("NISSEI ELECTRIC CO.,LTD");
		case ' NSP': return strdup("Nspire System Inc.");
		case ' NSS': return strdup("Newport Systems Solutions");
		case ' NST': return strdup("Network Security Technology Co");
		case ' NTC': return strdup("NeoTech S.R.L");
		case ' NTI': return strdup("New Tech Int'l Company");
		case ' NTL': return strdup("National Transcomm. Ltd");
		case ' NTN': return strdup("Nuvoton Technology Corporation");
		case ' NTR': return strdup("N-trig Innovative Technologies, Inc.");
		case ' NTS': return strdup("Nits Technology Inc.");
		case ' NTT': return strdup("NTT Advanced Technology Corporation");
		case ' NTW': return strdup("Networth Inc");
		case ' NTX': return strdup("Netaccess Inc");
		case ' NUG': return strdup("NU Technology, Inc.");
		case ' NUI': return strdup("NU Inc.");
		case ' NVC': return strdup("NetVision Corporation");
		case ' NVD': return strdup("Nvidia");
		case ' NVI': return strdup("NuVision US, Inc.");
		case ' NVL': return strdup("Novell Inc");
		case ' NVT': return strdup("Navatek Engineering Corporation");
		case ' NWC': return strdup("NW Computer Engineering");
		case ' NWP': return strdup("NovaWeb Technologies Inc");
		case ' NWS': return strdup("Newisys, Inc.");
		case ' NXC': return strdup("NextCom K.K.");
		case ' NXG': return strdup("Nexgen");
		case ' NXP': return strdup("NXP Semiconductors bv.");
		case ' NXQ': return strdup("Nexiq Technologies, Inc.");
		case ' NXS': return strdup("Technology Nexus Secure Open Systems AB");
		case ' NYC': return strdup("Nakayo Relecommunications, Inc.");
		case ' OAK': return strdup("Oak Tech Inc");
		case ' OAS': return strdup("Oasys Technology Company");
		case ' OBS': return strdup("Optibase Technologies");
		case ' OCD': return strdup("Macraigor Systems Inc");
		case ' OCN': return strdup("Olfan");
		case ' OCS': return strdup("Open Connect Solutions");
		case ' ODM': return strdup("ODME Inc.");
		case ' ODR': return strdup("Odrac");
		case ' OEC': return strdup("ORION ELECTRIC CO.,LTD");
		case ' OEI': return strdup("Optum Engineering Inc.");
		case ' OHW': return strdup("M-Labs Limited");
		case ' OIC': return strdup("Option Industrial Computers");
		case ' OIM': return strdup("Option International");
		case ' OIN': return strdup("Option International");
		case ' OKI': return strdup("OKI Electric Industrial Company Ltd");
		case ' OLC': return strdup("Olicom A/S");
		case ' OLD': return strdup("Olidata S.p.A.");
		case ' OLI': return strdup("Olivetti");
		case ' OLT': return strdup("Olitec S.A.");
		case ' OLV': return strdup("Olitec S.A.");
		case ' OLY': return strdup("OLYMPUS CORPORATION");
		case ' OMC': return strdup("OBJIX Multimedia Corporation");
		case ' OMN': return strdup("Omnitel");
		case ' OMR': return strdup("Omron Corporation");
		case ' ONE': return strdup("Oneac Corporation");
		case ' ONK': return strdup("ONKYO Corporation");
		case ' ONL': return strdup("OnLive, Inc");
		case ' ONS': return strdup("On Systems Inc");
		case ' ONW': return strdup("OPEN Networks Ltd");
		case ' ONX': return strdup("SOMELEC Z.I. Du Vert Galanta");
		case ' OOS': return strdup("OSRAM");
		case ' OPC': return strdup("Opcode Inc");
		case ' OPI': return strdup("D.N.S. Corporation");
		case ' OPP': return strdup("OPPO Digital, Inc.");
		case ' OPT': return strdup("OPTi Inc");
		case ' OPV': return strdup("Optivision Inc");
		case ' OQI': return strdup("Oksori Company Ltd");
		case ' ORG': return strdup("ORGA Kartensysteme GmbH");
		case ' ORI': return strdup("OSR Open Systems Resources, Inc.");
		case ' ORN': return strdup("ORION ELECTRIC CO., LTD.");
		case ' OSA': return strdup("OSAKA Micro Computer, Inc.");
		case ' OSD': return strdup("Optical Systems Design Pty Ltd");
		case ' OSI': return strdup("Open Stack, Inc.");
		case ' OSP': return strdup("OPTI-UPS Corporation");
		case ' OSR': return strdup("Oksori Company Ltd");
		case ' OTB': return strdup("outsidetheboxstuff.com");
		case ' OTI': return strdup("Orchid Technology");
		case ' OTK': return strdup("OmniTek");
		case ' OTM': return strdup("Optoma Corporation          ");
		case ' OTT': return strdup("OPTO22, Inc.");
		case ' OUK': return strdup("OUK Company Ltd");
		case ' OVR': return strdup("Oculus VR, Inc.");
		case ' OWL': return strdup("Mediacom Technologies Pte Ltd");
		case ' OXU': return strdup("Oxus Research S.A.");
		case ' OYO': return strdup("Shadow Systems");
		case ' OZC': return strdup("OZ Corporation");
		case ' OZO': return strdup("Tribe Computer Works Inc");
		case ' PAC': return strdup("Pacific Avionics Corporation");
		case ' PAD': return strdup("Promotion and Display Technology Ltd.");
		case ' PAK': return strdup("Many CNC System Co., Ltd.");
		case ' PAM': return strdup("Peter Antesberger Messtechnik");
		case ' PAN': return strdup("The Panda Project");
		case ' PAR': return strdup("Parallan Comp Inc");
		case ' PBI': return strdup("Pitney Bowes");
		case ' PBL': return strdup("Packard Bell Electronics");
		case ' PBN': return strdup("Packard Bell NEC");
		case ' PBV': return strdup("Pitney Bowes");
		case ' PCA': return strdup("Philips BU Add On Card");
		case ' PCB': return strdup("OCTAL S.A.");
		case ' PCC': return strdup("PowerCom Technology Company Ltd");
		case ' PCG': return strdup("First Industrial Computer Inc");
		case ' PCI': return strdup("Pioneer Computer Inc");
		case ' PCK': return strdup("PCBANK21");
		case ' PCL': return strdup("pentel.co.,ltd");
		case ' PCM': return strdup("PCM Systems Corporation");
		case ' PCO': return strdup("Performance Concepts Inc.,");
		case ' PCP': return strdup("Procomp USA Inc");
		case ' PCS': return strdup("TOSHIBA PERSONAL COMPUTER SYSTEM CORPRATION");
		case ' PCT': return strdup("PC-Tel Inc");
		case ' PCW': return strdup("Pacific CommWare Inc");
		case ' PCX': return strdup("PC Xperten");
		case ' PDM': return strdup("Psion Dacom Plc.");
		case ' PDN': return strdup("AT&T Paradyne");
		case ' PDR': return strdup("Pure Data Inc");
		case ' PDS': return strdup("PD Systems International Ltd");
		case ' PDT': return strdup("PDTS - Prozessdatentechnik und Systeme");
		case ' PDV': return strdup("Prodrive B.V.");
		case ' PEC': return strdup("POTRANS Electrical Corp.");
		case ' PEG': return strdup("Pegatron Corporation");
		case ' PEI': return strdup("PEI Electronics Inc");
		case ' PEL': return strdup("Primax Electric Ltd");
		case ' PEN': return strdup("Interactive Computer Products Inc");
		case ' PEP': return strdup("Peppercon AG");
		case ' PER': return strdup("Perceptive Signal Technologies");
		case ' PET': return strdup("Practical Electronic Tools");
		case ' PFT': return strdup("Telia ProSoft AB");
		case ' PGI': return strdup("PACSGEAR, Inc.");
		case ' PGM': return strdup("Paradigm Advanced Research Centre");
		case ' PGP': return strdup("propagamma kommunikation");
		case ' PGS': return strdup("Princeton Graphic Systems");
		case ' PHC': return strdup("Pijnenburg Beheer N.V.");
		case ' PHE': return strdup("Philips Medical Systems Boeblingen GmbH");
		case ' PHI': return strdup("DO NOT USE - PHI");
		case ' PHL': return strdup("Philips Consumer Electronics Company");
		case ' PHO': return strdup("Photonics Systems Inc.");
		case ' PHS': return strdup("Philips Communication Systems");
		case ' PHY': return strdup("Phylon Communications");
		case ' PIC': return strdup("Picturall Ltd.");
		case ' PIE': return strdup("Pacific Image Electronics Company Ltd");
		case ' PIM': return strdup("Prism, LLC");
		case ' PIO': return strdup("Pioneer Electronic Corporation");
		case ' PIX': return strdup("Pixie Tech Inc");
		case ' PJA': return strdup("Projecta");
		case ' PJD': return strdup("Projectiondesign AS");
		case ' PJT': return strdup("Pan Jit International Inc.");
		case ' PKA': return strdup("Acco UK Ltd.");
		case ' PLC': return strdup("Pro-Log Corporation");
		case ' PLF': return strdup("Panasonic Avionics Corporation");
		case ' PLM': return strdup("PROLINK Microsystems Corp.");
		case ' PLT': return strdup("PT Hartono Istana Teknologi");
		case ' PLV': return strdup("PLUS Vision Corp.");
		case ' PLX': return strdup("Parallax Graphics");
		case ' PLY': return strdup("Polycom Inc.");
		case ' PMC': return strdup("PMC Consumer Electronics Ltd");
		case ' PMD': return strdup("TDK USA Corporation");
		case ' PMM': return strdup("Point Multimedia System");
		case ' PMT': return strdup("Promate Electronic Co., Ltd.");
		case ' PMX': return strdup("Photomatrix");
		case ' PNG': return strdup("Microsoft");
		case ' PNL': return strdup("Panelview, Inc.");
		case ' PNP': return strdup("Microsoft");
		case ' PNR': return strdup("Planar Systems, Inc.");
		case ' PNS': return strdup("PanaScope");
		case ' PNX': return strdup("Phoenix Technologies, Ltd.");
		case ' POL': return strdup("PolyComp (PTY) Ltd.");
		case ' PON': return strdup("Perpetual Technologies, LLC");
		case ' POR': return strdup("Portalis LC");
		case ' POT': return strdup("Parrot");
		case ' PPC': return strdup("Phoenixtec Power Company Ltd");
		case ' PPD': return strdup("MEPhI");
		case ' PPI': return strdup("Practical Peripherals");
		case ' PPM': return strdup("Clinton Electronics Corp.");
		case ' PPP': return strdup("Purup Prepress AS");
		case ' PPR': return strdup("PicPro");
		case ' PPX': return strdup("Perceptive Pixel Inc.");
		case ' PQI': return strdup("Pixel Qi");
		case ' PRA': return strdup("PRO/AUTOMATION");
		case ' PRC': return strdup("PerComm");
		case ' PRD': return strdup("Praim S.R.L.");
		case ' PRF': return strdup("Digital Electronics Corporation");
		case ' PRG': return strdup("The Phoenix Research Group Inc");
		case ' PRI': return strdup("Priva Hortimation BV");
		case ' PRM': return strdup("Prometheus");
		case ' PRO': return strdup("Proteon");
		case ' PRP': return strdup("UEFI Forum");
		case ' PRS': return strdup("Leutron Vision");
		case ' PRT': return strdup("Parade Technologies, Ltd.");
		case ' PRX': return strdup("Proxima Corporation");
		case ' PSA': return strdup("Advanced Signal Processing Technologies");
		case ' PSC': return strdup("Philips Semiconductors");
		case ' PSD': return strdup("Peus-Systems GmbH");
		case ' PSE': return strdup("Practical Solutions Pte., Ltd.");
		case ' PSI': return strdup("PSI-Perceptive Solutions Inc");
		case ' PSL': return strdup("Perle Systems Limited");
		case ' PSM': return strdup("Prosum");
		case ' PST': return strdup("Global Data SA");
		case ' PSY': return strdup("Prodea Systems Inc.");
		case ' PTA': return strdup("PAR Tech Inc.");
		case ' PTC': return strdup("PS Technology Corporation");
		case ' PTG': return strdup("Cipher Systems Inc");
		case ' PTH': return strdup("Pathlight Technology Inc");
		case ' PTI': return strdup("Promise Technology Inc");
		case ' PTL': return strdup("Pantel Inc");
		case ' PTS': return strdup("Plain Tree Systems Inc");
		case ' PTW': return strdup("DO NOT USE - PTW");
		case ' PUL': return strdup("Pulse-Eight Ltd");
		case ' PVC': return strdup("DO NOT USE - PVC");
		case ' PVG': return strdup("Proview Global Co., Ltd");
		case ' PVI': return strdup("Prime view international Co., Ltd");
		case ' PVM': return strdup("Penta Studiotechnik GmbH");
		case ' PVN': return strdup("Pixel Vision");
		case ' PVP': return strdup("Klos Technologies, Inc.");
		case ' PXC': return strdup("Phoenix Contact");
		case ' PXE': return strdup("PIXELA CORPORATION");
		case ' PXL': return strdup("The Moving Pixel Company");
		case ' PXM': return strdup("Proxim Inc");
		case ' QCC': return strdup("QuakeCom Company Ltd");
		case ' QCH': return strdup("Metronics Inc");
		case ' QCI': return strdup("Quanta Computer Inc");
		case ' QCK': return strdup("Quick Corporation");
		case ' QCL': return strdup("Quadrant Components Inc");
		case ' QCP': return strdup("Qualcomm Inc");
		case ' QDI': return strdup("Quantum Data Incorporated");
		case ' QDM': return strdup("Quadram");
		case ' QDS': return strdup("Quanta Display Inc.");
		case ' QFF': return strdup("Padix Co., Inc.");
		case ' QFI': return strdup("Quickflex, Inc");
		case ' QLC': return strdup("Q-Logic");
		case ' QQQ': return strdup("Chuomusen Co., Ltd.");
		case ' QSI': return strdup("Quantum Solutions, Inc.");
		case ' QTD': return strdup("Quantum 3D Inc");
		case ' QTH': return strdup("Questech Ltd");
		case ' QTI': return strdup("Quicknet Technologies Inc");
		case ' QTM': return strdup("Quantum");
		case ' QTR': return strdup("Qtronix Corporation");
		case ' QUA': return strdup("Quatographic AG");
		case ' QUE': return strdup("Questra Consulting");
		case ' QVU': return strdup("Quartics");
		case ' RAC': return strdup("Racore Computer Products Inc");
		case ' RAD': return strdup("Radisys Corporation");
		case ' RAI': return strdup("Rockwell Automation/Intecolor");
		case ' RAN': return strdup("Rancho Tech Inc");
		case ' RAR': return strdup("Raritan, Inc.");
		case ' RAS': return strdup("RAScom Inc");
		case ' RAT': return strdup("Rent-A-Tech");
		case ' RAY': return strdup("Raylar Design, Inc.");
		case ' RCE': return strdup("Parc d'Activite des Bellevues");
		case ' RCH': return strdup("Reach Technology Inc");
		case ' RCI': return strdup("RC International");
		case ' RCN': return strdup("Radio Consult SRL");
		case ' RCO': return strdup("Rockwell Collins");
		case ' RDI': return strdup("Rainbow Displays, Inc.");
		case ' RDM': return strdup("Tremon Enterprises Company Ltd");
		case ' RDN': return strdup("RADIODATA GmbH");
		case ' RDS': return strdup("Radius Inc");
		case ' REA': return strdup("Real D");
		case ' REC': return strdup("ReCom");
		case ' RED': return strdup("Research Electronics Development Inc");
		case ' REF': return strdup("Reflectivity, Inc.");
		case ' REH': return strdup("Rehan Electronics Ltd.");
		case ' REL': return strdup("Reliance Electric Ind Corporation");
		case ' REM': return strdup("SCI Systems Inc.");
		case ' REN': return strdup("Renesas Technology Corp.");
		case ' RES': return strdup("ResMed Pty Ltd");
		case ' RET': return strdup("Resonance Technology, Inc.");
		case ' REV': return strdup("Revolution Display, Inc.");
		case ' REX': return strdup("RATOC Systems, Inc.");
		case ' RFI': return strdup("RAFI GmbH & Co. KG");
		case ' RFX': return strdup("Redfox Technologies Inc.");
		case ' RGB': return strdup("RGB Spectrum");
		case ' RGL': return strdup("Robertson Geologging Ltd");
		case ' RHD': return strdup("RightHand Technologies");
		case ' RHM': return strdup("Rohm Company Ltd");
		case ' RHT': return strdup("Red Hat, Inc.");
		case ' RIC': return strdup("RICOH COMPANY, LTD.");
		case ' RII': return strdup("Racal Interlan Inc");
		case ' RIO': return strdup("Rios Systems Company Ltd");
		case ' RIT': return strdup("Ritech Inc");
		case ' RIV': return strdup("Rivulet Communications");
		case ' RJA': return strdup("Roland Corporation");
		case ' RJS': return strdup("Advanced Engineering");
		case ' RKC': return strdup("Reakin Technolohy Corporation");
		case ' RLD': return strdup("MEPCO");
		case ' RLN': return strdup("RadioLAN Inc");
		case ' RMC': return strdup("Raritan Computer, Inc");
		case ' RMP': return strdup("Research Machines");
		case ' RMS': return strdup("Shenzhen Ramos Digital Technology Co., Ltd");
		case ' RMT': return strdup("Roper Mobile");
		case ' RNB': return strdup("Rainbow Technologies");
		case ' ROB': return strdup("Robust Electronics GmbH");
		case ' ROH': return strdup("Rohm Co., Ltd.");
		case ' ROK': return strdup("Rockwell International");
		case ' ROP': return strdup("Roper International Ltd");
		case ' ROS': return strdup("Rohde & Schwarz");
		case ' RPI': return strdup("RoomPro Technologies");
		case ' RPT': return strdup("R.P.T.Intergroups");
		case ' RRI': return strdup("Radicom Research Inc");
		case ' RSC': return strdup("PhotoTelesis");
		case ' RSH': return strdup("ADC-Centre");
		case ' RSI': return strdup("Rampage Systems Inc");
		case ' RSN': return strdup("Radiospire Networks, Inc.");
		case ' RSQ': return strdup("R Squared");
		case ' RSR': return strdup("Zhong Shan City Richsound Electronic Industrial Ltd.");
		case ' RSS': return strdup("Rockwell Semiconductor Systems");
		case ' RSV': return strdup("Ross Video Ltd");
		case ' RSX': return strdup("Rapid Tech Corporation");
		case ' RTC': return strdup("Relia Technologies");
		case ' RTI': return strdup("Rancho Tech Inc");
		case ' RTK': return strdup("DO NOT USE - RTK");
		case ' RTL': return strdup("Realtek Semiconductor Company Ltd");
		case ' RTS': return strdup("Raintree Systems");
		case ' RUN': return strdup("RUNCO International");
		case ' RUP': return strdup("Ups Manufactoring s.r.l.");
		case ' RVC': return strdup("RSI Systems Inc");
		case ' RVI': return strdup("Realvision Inc");
		case ' RVL': return strdup("Reveal Computer Prod");
		case ' RWC': return strdup("Red Wing Corporation");
		case ' RXT': return strdup("Tectona SoftSolutions (P) Ltd.,");
		case ' RZS': return strdup("Rozsnyó, s.r.o.");
		case ' SAA': return strdup("Sanritz Automation Co.,Ltd.");
		case ' SAE': return strdup("Saab Aerotech");
		case ' SAG': return strdup("Sedlbauer");
		case ' SAI': return strdup("Sage Inc");
		case ' SAK': return strdup("Saitek Ltd");
		case ' SAM': return strdup("Samsung Electric Company");
		case ' SAN': return strdup("Sanyo Electric Co.,Ltd.");
		case ' SAS': return strdup("Stores Automated Systems Inc");
		case ' SAT': return strdup("Shuttle Tech");
		case ' SBC': return strdup("Shanghai Bell Telephone Equip Mfg Co");
		case ' SBD': return strdup("Softbed - Consulting & Development Ltd");
		case ' SBI': return strdup("SMART Technologies Inc.");
		case ' SBS': return strdup("SBS-or Industrial Computers GmbH");
		case ' SBT': return strdup("Senseboard Technologies AB");
		case ' SCB': return strdup("SeeCubic B.V.");
		case ' SCC': return strdup("SORD Computer Corporation");
		case ' SCD': return strdup("Sanyo Electric Company Ltd");
		case ' SCE': return strdup("Sun Corporation");
		case ' SCH': return strdup("Schlumberger Cards");
		case ' SCI': return strdup("System Craft");
		case ' SCL': return strdup("Sigmacom Co., Ltd.");
		case ' SCM': return strdup("SCM Microsystems Inc");
		case ' SCN': return strdup("Scanport, Inc.");
		case ' SCO': return strdup("SORCUS Computer GmbH");
		case ' SCP': return strdup("Scriptel Corporation");
		case ' SCR': return strdup("Systran Corporation");
		case ' SCS': return strdup("Nanomach Anstalt");
		case ' SCT': return strdup("Smart Card Technology");
		case ' SCX': return strdup("Socionext Inc.");
		case ' SDA': return strdup("SAT (Societe Anonyme)");
		case ' SDD': return strdup("Intrada-SDD Ltd");
		case ' SDE': return strdup("Sherwood Digital Electronics Corporation");
		case ' SDF': return strdup("SODIFF E&T CO., Ltd.");
		case ' SDH': return strdup("Communications Specialies, Inc.");
		case ' SDI': return strdup("Samtron Displays Inc");
		case ' SDK': return strdup("SAIT-Devlonics");
		case ' SDR': return strdup("SDR Systems");
		case ' SDS': return strdup("SunRiver Data System");
		case ' SDT': return strdup("Siemens AG");
		case ' SDX': return strdup("SDX Business Systems Ltd");
		case ' SEA': return strdup("Seanix Technology Inc.");
		case ' SEB': return strdup("system elektronik GmbH");
		case ' SEC': return strdup("Seiko Epson Corporation");
		case ' SEE': return strdup("SeeColor Corporation");
		case ' SEG': return strdup("DO NOT USE - SEG");
		case ' SEI': return strdup("Seitz & Associates Inc");
		case ' SEL': return strdup("Way2Call Communications");
		case ' SEM': return strdup("Samsung Electronics Company Ltd");
		case ' SEN': return strdup("Sencore");
		case ' SEO': return strdup("SEOS Ltd");
		case ' SEP': return strdup("SEP Eletronica Ltda.");
		case ' SER': return strdup("Sony Ericsson Mobile Communications Inc.");
		case ' SES': return strdup("Session Control LLC");
		case ' SET': return strdup("SendTek Corporation");
		case ' SFM': return strdup("TORNADO Company");
		case ' SFT': return strdup("Mikroforum Ring 3");
		case ' SGC': return strdup("Spectragraphics Corporation");
		case ' SGD': return strdup("Sigma Designs, Inc.");
		case ' SGE': return strdup("Kansai Electric Company Ltd");
		case ' SGI': return strdup("Scan Group Ltd");
		case ' SGL': return strdup("Super Gate Technology Company Ltd");
		case ' SGM': return strdup("SAGEM");
		case ' SGO': return strdup("Logos Design A/S");
		case ' SGT': return strdup("Stargate Technology");
		case ' SGW': return strdup("Shanghai Guowei Science and Technology Co., Ltd.");
		case ' SGX': return strdup("Silicon Graphics Inc");
		case ' SGZ': return strdup("Systec Computer GmbH");
		case ' SHC': return strdup("ShibaSoku Co., Ltd.");
		case ' SHG': return strdup("Soft & Hardware development Goldammer GmbH");
		case ' SHI': return strdup("Jiangsu Shinco Electronic Group Co., Ltd");
		case ' SHP': return strdup("Sharp Corporation");
		case ' SHR': return strdup("Digital Discovery");
		case ' SHT': return strdup("Shin Ho Tech");
		case ' SIA': return strdup("SIEMENS AG");
		case ' SIB': return strdup("Sanyo Electric Company Ltd");
		case ' SIC': return strdup("Sysmate Corporation");
		case ' SID': return strdup("Seiko Instruments Information Devices Inc");
		case ' SIE': return strdup("Siemens");
		case ' SIG': return strdup("Sigma Designs Inc");
		case ' SII': return strdup("Silicon Image, Inc.");
		case ' SIL': return strdup("Silicon Laboratories, Inc");
		case ' SIM': return strdup("S3 Inc");
		case ' SIN': return strdup("Singular Technology Co., Ltd.");
		case ' SIR': return strdup("Sirius Technologies Pty Ltd");
		case ' SIS': return strdup("Silicon Integrated Systems Corporation");
		case ' SIT': return strdup("Sitintel");
		case ' SIU': return strdup("Seiko Instruments USA Inc");
		case ' SIX': return strdup("Zuniq Data Corporation");
		case ' SJE': return strdup("Sejin Electron Inc");
		case ' SKD': return strdup("Schneider & Koch");
		case ' SKM': return strdup("Guangzhou Teclast Information Technology Limited");
		case ' SKT': return strdup("Samsung Electro-Mechanics Company Ltd");
		case ' SKY': return strdup("SKYDATA S.P.A.");
		case ' SLA': return strdup("Systeme Lauer GmbH&Co KG");
		case ' SLB': return strdup("Shlumberger Ltd");
		case ' SLC': return strdup("Syslogic Datentechnik AG");
		case ' SLF': return strdup("StarLeaf");
		case ' SLH': return strdup("Silicon Library Inc.");
		case ' SLI': return strdup("Symbios Logic Inc");
		case ' SLK': return strdup("Silitek Corporation");
		case ' SLM': return strdup("Solomon Technology Corporation");
		case ' SLR': return strdup("Schlumberger Technology Corporate");
		case ' SLS': return strdup("Schnick-Schnack-Systems GmbH");
		case ' SLT': return strdup("Salt Internatioinal Corp.");
		case ' SLX': return strdup("Specialix");
		case ' SMA': return strdup("SMART Modular Technologies");
		case ' SMB': return strdup("Schlumberger");
		case ' SMC': return strdup("Standard Microsystems Corporation");
		case ' SME': return strdup("Sysmate Company");
		case ' SMI': return strdup("SpaceLabs Medical Inc");
		case ' SMK': return strdup("SMK CORPORATION");
		case ' SML': return strdup("Sumitomo Metal Industries, Ltd.");
		case ' SMM': return strdup("Shark Multimedia Inc");
		case ' SMO': return strdup("STMicroelectronics");
		case ' SMP': return strdup("Simple Computing");
		case ' SMR': return strdup("B.& V. s.r.l.");
		case ' SMS': return strdup("Silicom Multimedia Systems Inc");
		case ' SMT': return strdup("Silcom Manufacturing Tech Inc");
		case ' SNC': return strdup("Sentronic International Corp.");
		case ' SNI': return strdup("Siemens Microdesign GmbH");
		case ' SNK': return strdup("S&K Electronics");
		case ' SNN': return strdup("SUNNY ELEKTRONIK");
		case ' SNO': return strdup("SINOSUN TECHNOLOGY CO., LTD");
		case ' SNP': return strdup("Siemens Nixdorf Info Systems");
		case ' SNS': return strdup("Cirtech (UK) Ltd");
		case ' SNT': return strdup("SuperNet Inc");
		case ' SNW': return strdup("Snell & Wilcox");
		case ' SNX': return strdup("Sonix Comm. Ltd");
		case ' SNY': return strdup("Sony");
		case ' SOC': return strdup("Santec Corporation");
		case ' SOI': return strdup("Silicon Optix Corporation");
		case ' SOL': return strdup("Solitron Technologies Inc");
		case ' SON': return strdup("Sony");
		case ' SOR': return strdup("Sorcus Computer GmbH");
		case ' SOT': return strdup("Sotec Company Ltd");
		case ' SOY': return strdup("SOYO Group, Inc");
		case ' SPC': return strdup("SpinCore Technologies, Inc");
		case ' SPE': return strdup("SPEA Software AG");
		case ' SPH': return strdup("G&W Instruments GmbH");
		case ' SPI': return strdup("SPACE-I Co., Ltd.");
		case ' SPK': return strdup("SpeakerCraft");
		case ' SPL': return strdup("Smart Silicon Systems Pty Ltd");
		case ' SPN': return strdup("Sapience Corporation");
		case ' SPR': return strdup("pmns GmbH");
		case ' SPS': return strdup("Synopsys Inc");
		case ' SPT': return strdup("Sceptre Tech Inc");
		case ' SPU': return strdup("SIM2 Multimedia S.P.A.");
		case ' SPX': return strdup("Simplex Time Recorder Co.");
		case ' SQT': return strdup("Sequent Computer Systems Inc");
		case ' SRC': return strdup("Integrated Tech Express Inc");
		case ' SRD': return strdup("Setred");
		case ' SRF': return strdup("Surf Communication Solutions Ltd");
		case ' SRG': return strdup("Intuitive Surgical, Inc.");
		case ' SRS': return strdup("SR-Systems e.K.");
		case ' SRT': return strdup("SeeReal Technologies GmbH");
		case ' SSC': return strdup("Sierra Semiconductor Inc");
		case ' SSD': return strdup("FlightSafety International");
		case ' SSE': return strdup("Samsung Electronic Co.");
		case ' SSI': return strdup("S-S Technology Inc");
		case ' SSJ': return strdup("Sankyo Seiki Mfg.co., Ltd");
		case ' SSL': return strdup("Shenzhen South-Top Computer Co., Ltd.");
		case ' SSP': return strdup("Spectrum Signal Proecessing Inc");
		case ' SSS': return strdup("S3 Inc");
		case ' SST': return strdup("SystemSoft Corporation");
		case ' STA': return strdup("ST Electronics Systems Assembly Pte Ltd");
		case ' STB': return strdup("STB Systems Inc");
		case ' STC': return strdup("STAC Electronics");
		case ' STD': return strdup("STD Computer Inc");
		case ' STE': return strdup("SII Ido-Tsushin Inc");
		case ' STF': return strdup("Starflight Electronics");
		case ' STG': return strdup("StereoGraphics Corp.");
		case ' STH': return strdup("Semtech Corporation");
		case ' STI': return strdup("Smart Tech Inc");
		case ' STK': return strdup("SANTAK CORP.");
		case ' STL': return strdup("SigmaTel Inc");
		case ' STM': return strdup("SGS Thomson Microelectronics");
		case ' STN': return strdup("Samsung Electronics America");
		case ' STO': return strdup("Stollmann E+V GmbH");
		case ' STP': return strdup("StreamPlay Ltd");
		case ' STQ': return strdup("Synthetel Corporation");
		case ' STR': return strdup("Starlight Networks Inc");
		case ' STS': return strdup("SITECSYSTEM CO., LTD.");
		case ' STT': return strdup("Star Paging Telecom Tech (Shenzhen) Co. Ltd.");
		case ' STU': return strdup("Sentelic Corporation");
		case ' STW': return strdup("Starwin Inc.");
		case ' STX': return strdup("ST-Ericsson");
		case ' STY': return strdup("SDS Technologies");
		case ' SUB': return strdup("Subspace Comm. Inc");
		case ' SUM': return strdup("Summagraphics Corporation");
		case ' SUN': return strdup("Sun Electronics Corporation");
		case ' SUP': return strdup("Supra Corporation");
		case ' SUR': return strdup("Surenam Computer Corporation");
		case ' SVA': return strdup("SGEG");
		case ' SVC': return strdup("Intellix Corp.");
		case ' SVD': return strdup("SVD Computer");
		case ' SVI': return strdup("Sun Microsystems");
		case ' SVR': return strdup("Sensics, Inc.");
		case ' SVS': return strdup("SVSI");
		case ' SVT': return strdup("SEVIT Co., Ltd.");
		case ' SWC': return strdup("Software Café");
		case ' SWI': return strdup("Sierra Wireless Inc.");
		case ' SWL': return strdup("Sharedware Ltd");
		case ' SWO': return strdup("Guangzhou Shirui Electronics Co., Ltd.");
		case ' SWS': return strdup("Static");
		case ' SWT': return strdup("Software Technologies Group,Inc.");
		case ' SXB': return strdup("Syntax-Brillian");
		case ' SXD': return strdup("Silex technology, Inc.");
		case ' SXG': return strdup("SELEX GALILEO");
		case ' SXL': return strdup("SolutionInside");
		case ' SXT': return strdup("SHARP TAKAYA ELECTRONIC INDUSTRY CO.,LTD.");
		case ' SYC': return strdup("Sysmic");
		case ' SYE': return strdup("SY Electronics Ltd");
		case ' SYK': return strdup("Stryker Communications");
		case ' SYL': return strdup("Sylvania Computer Products");
		case ' SYM': return strdup("Symicron Computer Communications Ltd.");
		case ' SYN': return strdup("Synaptics Inc");
		case ' SYP': return strdup("SYPRO Co Ltd");
		case ' SYS': return strdup("Sysgration Ltd");
		case ' SYT': return strdup("Seyeon Tech Company Ltd");
		case ' SYV': return strdup("SYVAX Inc");
		case ' SYX': return strdup("Prime Systems, Inc.");
		case ' SZM': return strdup("Shenzhen MTC Co., Ltd");
		case ' TAA': return strdup("Tandberg");
		case ' TAB': return strdup("Todos Data System AB");
		case ' TAG': return strdup("Teles AG");
		case ' TAI': return strdup("Toshiba America Info Systems Inc");
		case ' TAM': return strdup("Tamura Seisakusyo Ltd");
		case ' TAS': return strdup("Taskit Rechnertechnik GmbH");
		case ' TAT': return strdup("Teleliaison Inc");
		case ' TAV': return strdup("Thales Avionics");
		case ' TAX': return strdup("Taxan (Europe) Ltd");
		case ' TBB': return strdup("Triple S Engineering Inc");
		case ' TBC': return strdup("Turbo Communication, Inc");
		case ' TBS': return strdup("Turtle Beach System");
		case ' TCC': return strdup("Tandon Corporation");
		case ' TCD': return strdup("Taicom Data Systems Co., Ltd.");
		case ' TCE': return strdup("Century Corporation");
		case ' TCH': return strdup("Interaction Systems, Inc");
		case ' TCI': return strdup("Tulip Computers Int'l B.V.");
		case ' TCJ': return strdup("TEAC America Inc");
		case ' TCL': return strdup("Technical Concepts Ltd");
		case ' TCM': return strdup("3Com Corporation");
		case ' TCN': return strdup("Tecnetics (PTY) Ltd");
		case ' TCO': return strdup("Thomas-Conrad Corporation");
		case ' TCR': return strdup("Thomson Consumer Electronics");
		case ' TCS': return strdup("Tatung Company of America Inc");
		case ' TCT': return strdup("Telecom Technology Centre Co. Ltd.");
		case ' TCX': return strdup("FREEMARS Heavy Industries");
		case ' TDC': return strdup("Teradici");
		case ' TDD': return strdup("Tandberg Data Display AS");
		case ' TDG': return strdup("Six15 Technologies");
		case ' TDM': return strdup("Tandem Computer Europe Inc");
		case ' TDP': return strdup("3D Perception");
		case ' TDS': return strdup("Tri-Data Systems Inc");
		case ' TDT': return strdup("TDT");
		case ' TDV': return strdup("TDVision Systems, Inc.");
		case ' TDY': return strdup("Tandy Electronics");
		case ' TEA': return strdup("TEAC System Corporation");
		case ' TEC': return strdup("Tecmar Inc");
		case ' TEK': return strdup("Tektronix Inc");
		case ' TEL': return strdup("Promotion and Display Technology Ltd.");
		case ' TER': return strdup("TerraTec Electronic GmbH");
		case ' TET': return strdup("TETRADYNE CO., LTD.");
		case ' TEZ': return strdup("Tech Source Inc.");
		case ' TGC': return strdup("Toshiba Global Commerce Solutions, Inc.");
		case ' TGI': return strdup("TriGem Computer Inc");
		case ' TGM': return strdup("TriGem Computer,Inc.");
		case ' TGS': return strdup("Torus Systems Ltd");
		case ' TGV': return strdup("Grass Valley Germany GmbH");
		case ' THN': return strdup("Thundercom Holdings Sdn. Bhd.");
		case ' TIC': return strdup("Trigem KinfoComm");
		case ' TIL': return strdup("Technical Illusions Inc.");
		case ' TIP': return strdup("TIPTEL AG");
		case ' TIV': return strdup("OOO Technoinvest");
		case ' TIX': return strdup("Tixi.Com GmbH");
		case ' TKC': return strdup("Taiko Electric Works.LTD");
		case ' TKG': return strdup("Tek Gear");
		case ' TKN': return strdup("Teknor Microsystem Inc");
		case ' TKO': return strdup("TouchKo, Inc.");
		case ' TKS': return strdup("TimeKeeping Systems, Inc.");
		case ' TLA': return strdup("Ferrari Electronic GmbH");
		case ' TLD': return strdup("Telindus");
		case ' TLE': return strdup("Zhejiang Tianle Digital Electric Co., Ltd.");
		case ' TLF': return strdup("Teleforce.,co,ltd");
		case ' TLI': return strdup("TOSHIBA TELI CORPORATION");
		case ' TLK': return strdup("Telelink AG");
		case ' TLL': return strdup("Thinklogical");
		case ' TLS': return strdup("Teleste Educational OY");
		case ' TLT': return strdup("Dai Telecom S.p.A.");
		case ' TLV': return strdup("S3 Inc");
		case ' TLX': return strdup("Telxon Corporation");
		case ' TMC': return strdup("Techmedia Computer Systems Corporation");
		case ' TME': return strdup("AT&T Microelectronics");
		case ' TMI': return strdup("Texas Microsystem");
		case ' TMM': return strdup("Time Management, Inc.");
		case ' TMR': return strdup("Taicom International Inc");
		case ' TMS': return strdup("Trident Microsystems Ltd");
		case ' TMT': return strdup("T-Metrics Inc.");
		case ' TMX': return strdup("Thermotrex Corporation");
		case ' TNC': return strdup("TNC Industrial Company Ltd");
		case ' TNJ': return strdup("DO NOT USE - TNJ");
		case ' TNM': return strdup("TECNIMAGEN SA");
		case ' TNY': return strdup("Tennyson Tech Pty Ltd");
		case ' TOE': return strdup("TOEI Electronics Co., Ltd.");
		case ' TOG': return strdup("The OPEN Group");
		case ' TOL': return strdup("TCL Corporation");
		case ' TOM': return strdup("Ceton Corporation");
		case ' TON': return strdup("TONNA");
		case ' TOP': return strdup("Orion Communications Co., Ltd.");
		case ' TOS': return strdup("Toshiba Corporation");
		case ' TOU': return strdup("Touchstone Technology");
		case ' TPC': return strdup("Touch Panel Systems Corporation");
		case ' TPD': return strdup("Times (Shanghai) Computer Co., Ltd.");
		case ' TPE': return strdup("Technology Power Enterprises Inc");
		case ' TPJ': return strdup("Junnila");
		case ' TPK': return strdup("TOPRE CORPORATION");
		case ' TPR': return strdup("Topro Technology Inc");
		case ' TPS': return strdup("Teleprocessing Systeme GmbH");
		case ' TPT': return strdup("Thruput Ltd");
		case ' TPV': return strdup("Top Victory Electronics ( Fujian ) Company Ltd");
		case ' TPZ': return strdup("Ypoaz Systems Inc");
		case ' TRA': return strdup("TriTech Microelectronics International");
		case ' TRB': return strdup("Triumph Board a.s.");
		case ' TRC': return strdup("Trioc AB");
		case ' TRD': return strdup("Trident Microsystem Inc");
		case ' TRE': return strdup("Tremetrics");
		case ' TRI': return strdup("Tricord Systems");
		case ' TRL': return strdup("Royal Information");
		case ' TRM': return strdup("Tekram Technology Company Ltd");
		case ' TRN': return strdup("Datacommunicatie Tron B.V.");
		case ' TRS': return strdup("Torus Systems Ltd");
		case ' TRT': return strdup("Tritec Electronic AG");
		case ' TRU': return strdup("Aashima Technology B.V.");
		case ' TRV': return strdup("Trivisio Prototyping GmbH");
		case ' TRX': return strdup("Trex Enterprises");
		case ' TSB': return strdup("Toshiba America Info Systems Inc");
		case ' TSC': return strdup("Sanyo Electric Company Ltd");
		case ' TSD': return strdup("TechniSat Digital GmbH");
		case ' TSE': return strdup("Tottori Sanyo Electric");
		case ' TSF': return strdup("Racal-Airtech Software Forge Ltd");
		case ' TSG': return strdup("The Software Group Ltd");
		case ' TSH': return strdup("ELAN MICROELECTRONICS CORPORATION");
		case ' TSI': return strdup("TeleVideo Systems");
		case ' TSL': return strdup("Tottori SANYO Electric Co., Ltd.");
		case ' TSP': return strdup("U.S. Navy");
		case ' TST': return strdup("Transtream Inc");
		case ' TSV': return strdup("TRANSVIDEO");
		case ' TSY': return strdup("TouchSystems");
		case ' TTA': return strdup("Topson Technology Co., Ltd.");
		case ' TTB': return strdup("National Semiconductor Japan Ltd");
		case ' TTC': return strdup("Telecommunications Techniques Corporation");
		case ' TTE': return strdup("TTE, Inc.");
		case ' TTI': return strdup("Trenton Terminals Inc");
		case ' TTK': return strdup("Totoku Electric Company Ltd");
		case ' TTL': return strdup("2-Tel B.V");
		case ' TTP': return strdup("Toshiba Corporation");
		case ' TTS': return strdup("TechnoTrend Systemtechnik GmbH");
		case ' TTX': return strdup("Taitex Corporation");
		case ' TTY': return strdup("TRIDELITY Display Solutions GmbH");
		case ' TUA': return strdup("T+A elektroakustik GmbH");
		case ' TUT': return strdup("Tut Systems");
		case ' TVD': return strdup("Tecnovision");
		case ' TVI': return strdup("Truevision");
		case ' TVM': return strdup("Taiwan Video & Monitor Corporation");
		case ' TVO': return strdup("TV One Ltd");
		case ' TVR': return strdup("TV Interactive Corporation");
		case ' TVS': return strdup("TVS Electronics Limited");
		case ' TVV': return strdup("TV1 GmbH");
		case ' TWA': return strdup("Tidewater Association");
		case ' TWE': return strdup("Kontron Electronik");
		case ' TWH': return strdup("Twinhead International Corporation");
		case ' TWI': return strdup("Easytel oy");
		case ' TWK': return strdup("TOWITOKO electronics GmbH");
		case ' TWX': return strdup("TEKWorx Limited");
		case ' TXL': return strdup("Trixel Ltd");
		case ' TXN': return strdup("Texas Insturments");
		case ' TXT': return strdup("Textron Defense System");
		case ' TYN': return strdup("Tyan Computer Corporation");
		case ' UAS': return strdup("Ultima Associates Pte Ltd");
		case ' UBI': return strdup("Ungermann-Bass Inc");
		case ' UBL': return strdup("Ubinetics Ltd.");
		case ' UBU': return strdup("Canonical Ltd.");
		case ' UDN': return strdup("Uniden Corporation");
		case ' UEC': return strdup("Ultima Electronics Corporation");
		case ' UEG': return strdup("Elitegroup Computer Systems Company Ltd");
		case ' UEI': return strdup("Universal Electronics Inc");
		case ' UET': return strdup("Universal Empowering Technologies");
		case ' UFG': return strdup("UNIGRAF-USA");
		case ' UFO': return strdup("UFO Systems Inc");
		case ' UHB': return strdup("XOCECO");
		case ' UIC': return strdup("Uniform Industrial Corporation");
		case ' UJR': return strdup("Ueda Japan Radio Co., Ltd.");
		case ' ULT': return strdup("Ultra Network Tech");
		case ' UMC': return strdup("United Microelectr Corporation");
		case ' UMG': return strdup("Umezawa Giken Co.,Ltd");
		case ' UMM': return strdup("Universal Multimedia");
		case ' UMT': return strdup("UltiMachine");
		case ' UNA': return strdup("Unisys DSD");
		case ' UNB': return strdup("Unisys Corporation");
		case ' UNC': return strdup("Unisys Corporation");
		case ' UND': return strdup("DO NOT USE - UND");
		case ' UNE': return strdup("DO NOT USE - UNE");
		case ' UNF': return strdup("DO NOT USE - UNF");
		case ' UNI': return strdup("Uniform Industry Corp.");
		case ' UNM': return strdup("Unisys Corporation");
		case ' UNO': return strdup("Unisys Corporation");
		case ' UNP': return strdup("Unitop");
		case ' UNS': return strdup("Unisys Corporation");
		case ' UNT': return strdup("Unisys Corporation");
		case ' UNY': return strdup("Unicate");
		case ' UPP': return strdup("UPPI");
		case ' UPS': return strdup("Systems Enhancement");
		case ' URD': return strdup("Video Computer S.p.A.");
		case ' USA': return strdup("Utimaco Safeware AG");
		case ' USD': return strdup("U.S. Digital Corporation");
		case ' USE': return strdup("U. S. Electronics Inc.");
		case ' USI': return strdup("Universal Scientific Industrial Co., Ltd.");
		case ' USR': return strdup("U.S. Robotics Inc");
		case ' UTD': return strdup("Up to Date Tech");
		case ' UWC': return strdup("Uniwill Computer Corp.");
		case ' VAD': return strdup("Vaddio, LLC");
		case ' VAI': return strdup("VAIO Corporation");
		case ' VAL': return strdup("Valence Computing Corporation");
		case ' VAR': return strdup("Varian Australia Pty Ltd");
		case ' VBR': return strdup("VBrick Systems Inc.");
		case ' VBT': return strdup("Valley Board Ltda");
		case ' VCC': return strdup("Virtual Computer Corporation");
		case ' VCI': return strdup("VistaCom Inc");
		case ' VCJ': return strdup("Victor Company of Japan, Limited");
		case ' VCM': return strdup("Vector Magnetics, LLC");
		case ' VCX': return strdup("VCONEX");
		case ' VDA': return strdup("Victor Data Systems");
		case ' VDC': return strdup("VDC Display Systems");
		case ' VDM': return strdup("Vadem");
		case ' VDO': return strdup("Video & Display Oriented Corporation");
		case ' VDS': return strdup("Vidisys GmbH & Company");
		case ' VDT': return strdup("Viditec, Inc.");
		case ' VEC': return strdup("Vector Informatik GmbH");
		case ' VEK': return strdup("Vektrex");
		case ' VES': return strdup("Vestel Elektronik Sanayi ve Ticaret A. S.");
		case ' VFI': return strdup("VeriFone Inc");
		case ' VHI': return strdup("Macrocad Development Inc.");
		case ' VIA': return strdup("VIA Tech Inc");
		case ' VIB': return strdup("Tatung UK Ltd");
		case ' VIC': return strdup("Victron B.V.");
		case ' VID': return strdup("Ingram Macrotron Germany");
		case ' VIK': return strdup("Viking Connectors");
		case ' VIM': return strdup("Via Mons Ltd.");
		case ' VIN': return strdup("Vine Micros Ltd");
		case ' VIR': return strdup("Visual Interface, Inc");
		case ' VIS': return strdup("Visioneer");
		case ' VIT': return strdup("Visitech AS");
		case ' VIZ': return strdup("VIZIO, Inc");
		case ' VLB': return strdup("ValleyBoard Ltda.");
		case ' VLC': return strdup("VersaLogic Corporation");
		case ' VLK': return strdup("Vislink International Ltd");
		case ' VLT': return strdup("VideoLan Technologies");
		case ' VLV': return strdup("Valve Corporation");
		case ' VMI': return strdup("Vermont Microsystems");
		case ' VML': return strdup("Vine Micros Limited");
		case ' VMW': return strdup("VMware Inc.,");
		case ' VNC': return strdup("Vinca Corporation");
		case ' VOB': return strdup("MaxData Computer AG");
		case ' VPI': return strdup("Video Products Inc");
		case ' VPR': return strdup("Best Buy");
		case ' VPX': return strdup("VPixx Technologies Inc.");
		case ' VQ@': return strdup("Vision Quest");
		case ' VRC': return strdup("Virtual Resources Corporation");
		case ' VRM': return strdup("VRmagic Holding AG");
		case ' VSC': return strdup("ViewSonic Corporation");
		case ' VSD': return strdup("3M");
		case ' VSI': return strdup("VideoServer");
		case ' VSN': return strdup("Ingram Macrotron");
		case ' VSP': return strdup("Vision Systems GmbH");
		case ' VSR': return strdup("V-Star Electronics Inc.");
		case ' VTB': return strdup("Videotechnik Breithaupt");
		case ' VTC': return strdup("VTel Corporation");
		case ' VTG': return strdup("Voice Technologies Group Inc");
		case ' VTI': return strdup("VLSI Tech Inc");
		case ' VTK': return strdup("Viewteck Co., Ltd.");
		case ' VTL': return strdup("Vivid Technology Pte Ltd");
		case ' VTM': return strdup("Miltope Corporation");
		case ' VTN': return strdup("VIDEOTRON CORP.");
		case ' VTS': return strdup("VTech Computers Ltd");
		case ' VTV': return strdup("VATIV Technologies");
		case ' VTX': return strdup("Vestax Corporation");
		case ' VUT': return strdup("Vutrix (UK) Ltd");
		case ' VWB': return strdup("Vweb Corp.");
		case ' WAC': return strdup("Wacom Tech");
		case ' WAL': return strdup("Wave Access");
		case ' WAN': return strdup("DO NOT USE - WAN");
		case ' WAV': return strdup("Wavephore");
		case ' WBN': return strdup("MicroSoftWare");
		case ' WBS': return strdup("WB Systemtechnik GmbH");
		case ' WCI': return strdup("Wisecom Inc");
		case ' WCS': return strdup("Woodwind Communications Systems Inc");
		case ' WDC': return strdup("Western Digital");
		case ' WDE': return strdup("Westinghouse Digital Electronics");
		case ' WEB': return strdup("WebGear Inc");
		case ' WEC': return strdup("Winbond Electronics Corporation");
		case ' WEL': return strdup("W-DEV");
		case ' WEY': return strdup("WEY Design AG");
		case ' WHI': return strdup("Whistle Communications");
		case ' WII': return strdup("Innoware Inc");
		case ' WIL': return strdup("WIPRO Information Technology Ltd");
		case ' WIN': return strdup("Wintop Technology Inc");
		case ' WIP': return strdup("Wipro Infotech");
		case ' WKH': return strdup("Uni-Take Int'l Inc.");
		case ' WLD': return strdup("Wildfire Communications Inc");
		case ' WML': return strdup("Wolfson Microelectronics Ltd");
		case ' WMO': return strdup("Westermo Teleindustri AB");
		case ' WMT': return strdup("Winmate Communication Inc");
		case ' WNI': return strdup("WillNet Inc.");
		case ' WNV': return strdup("Winnov L.P.");
		case ' WNX': return strdup("Wincor Nixdorf International GmbH");
		case ' WPA': return strdup("Matsushita Communication Industrial Co., Ltd.");
		case ' WPI': return strdup("Wearnes Peripherals International (Pte) Ltd");
		case ' WRC': return strdup("WiNRADiO Communications");
		case ' WSC': return strdup("CIS Technology Inc");
		case ' WSP': return strdup("Wireless And Smart Products Inc.");
		case ' WST': return strdup("Wistron Corporation");
		case ' WTC': return strdup("ACC Microelectronics");
		case ' WTI': return strdup("WorkStation Tech");
		case ' WTK': return strdup("Wearnes Thakral Pte");
		case ' WTS': return strdup("Restek Electric Company Ltd");
		case ' WVM': return strdup("Wave Systems Corporation");
		case ' WVV': return strdup("WolfVision GmbH");
		case ' WWP': return strdup("Wipotec Wiege- und Positioniersysteme GmbH");
		case ' WWV': return strdup("World Wide Video, Inc.");
		case ' WXT': return strdup("Woxter Technology Co. Ltd");
		case ' WYS': return strdup("Myse Technology");
		case ' WYT': return strdup("Wooyoung Image & Information Co.,Ltd.");
		case ' XAC': return strdup("XAC Automation Corp");
		case ' XAD': return strdup("Alpha Data");
		case ' XDM': return strdup("XDM Ltd.");
		case ' XER': return strdup("DO NOT USE - XER");
		case ' XFG': return strdup("Jan Strapko - FOTO");
		case ' XFO': return strdup("EXFO Electro Optical Engineering");
		case ' XIN': return strdup("Xinex Networks Inc");
		case ' XIO': return strdup("Xiotech Corporation");
		case ' XIR': return strdup("Xirocm Inc");
		case ' XIT': return strdup("Xitel Pty ltd");
		case ' XLX': return strdup("Xilinx, Inc.");
		case ' XMM': return strdup("C3PO S.L.");
		case ' XNT': return strdup("XN Technologies, Inc.");
		case ' XOC': return strdup("DO NOT USE - XOC");
		case ' XQU': return strdup("SHANGHAI SVA-DAV ELECTRONICS CO., LTD");
		case ' XRC': return strdup("Xircom Inc");
		case ' XRO': return strdup("XORO ELECTRONICS (CHENGDU) LIMITED");
		case ' XSN': return strdup("Xscreen AS");
		case ' XST': return strdup("XS Technologies Inc");
		case ' XSY': return strdup("XSYS");
		case ' XTD': return strdup("Icuiti Corporation");
		case ' XTE': return strdup("X2E GmbH");
		case ' XTL': return strdup("Crystal Computer");
		case ' XTN': return strdup("X-10 (USA) Inc");
		case ' XYC': return strdup("Xycotec Computer GmbH");
		case ' XYE': return strdup("Shenzhen Zhuona Technology Co., Ltd.");
		case ' YED': return strdup("Y-E Data Inc");
		case ' YHQ': return strdup("Yokogawa Electric Corporation");
		case ' YHW': return strdup("Exacom SA");
		case ' YMH': return strdup("Yamaha Corporation");
		case ' YOW': return strdup("American Biometric Company");
		case ' ZAN': return strdup("Zandar Technologies plc");
		case ' ZAX': return strdup("Zefiro Acoustics");
		case ' ZAZ': return strdup("Zazzle Technologies");
		case ' ZBR': return strdup("Zebra Technologies International, LLC");
		case ' ZBX': return strdup("Zebax Technologies");
		case ' ZCT': return strdup("ZeitControl cardsystems GmbH");
		case ' ZDS': return strdup("Zenith Data Systems");
		case ' ZEN': return strdup("ZENIC Inc.");
		case ' ZGT': return strdup("Zenith Data Systems");
		case ' ZIC': return strdup("Nationz Technologies Inc.");
		case ' ZMC': return strdup("HangZhou ZMCHIVIN");
		case ' ZMT': return strdup("Zalman Tech Co., Ltd.");
		case ' ZMZ': return strdup("Z Microsystems");
		case ' ZNI': return strdup("Zetinet Inc");
		case ' ZNX': return strdup("Znyx Adv. Systems");
		case ' ZOW': return strdup("Zowie Intertainment, Inc");
		case ' ZRN': return strdup("Zoran Corporation");
		case ' ZSE': return strdup("Zenith Data Systems");
		case ' ZTC': return strdup("ZyDAS Technology Corporation");
		case ' ZTE': return strdup("ZTE Corporation");
		case ' ZTI': return strdup("Zoom Telephonics Inc");
		case ' ZTM': return strdup("ZT Group Int'l Inc.");
		case ' ZTT': return strdup("Z3 Technology");
		case ' ZWE': return strdup("Shenzhen Zowee Technology Co., LTD");
		case ' ZYD': return strdup("Zydacron Inc");
		case ' ZYP': return strdup("Zypcom Inc");
		case ' ZYT': return strdup("Zytex Computers");
		case ' ZYX': return strdup("Zyxel");
		case ' ZZZ': return strdup("Boca Research Inc");
		// ========================================================


		// Not listed in the PNP ID registry:
		case ' DIP': return strdup("Duet Display");
		case ' HOP': return strdup("HP Compaq");


		default:
			return VuoText_format("%c%c%c", fourCC >> 16, (fourCC >> 8) & 0xff, fourCC & 0xff);
	}
}
